﻿using Microsoft.IdentityModel.Clients.ActiveDirectory;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.Samples.AuthenticationManagement
{
    /// <summary>
    /// Class for managing auth token cache
    /// </summary>
    public class AuthTokenCache : TokenCache
    {
        private Dictionary<string, AuthTokenCacheInfo> caches;

        public AuthTokenCache(string state = null)
        {
            if (state == null)
            {
                caches = new Dictionary<string, AuthTokenCacheInfo>();
            }
            else
            {
                caches = JsonConvert.DeserializeObject<Dictionary<string, AuthTokenCacheInfo>>(state);
            }
        }

        public IEnumerable<AuthTokenCacheInfo> GetValues(string resource)
        {
            return caches.Values.Where(c => c.Resource == resource);
        }

        public string GetState()
        {
            return JObject.FromObject(caches).ToString();
        }

        public bool TryGetValue(string tenantId, string resource, LoginType loginType, out AuthTokenCacheInfo cacheInfo)
        {
            return caches.TryGetValue(GetKey(tenantId, resource, loginType), out cacheInfo);
        }

        public AuthTokenCacheInfo Get(string tenantId, string resource, LoginType loginType)
        {
            return caches[GetKey(tenantId, resource, loginType)];
        }

        public void Add(AuthTokenCacheInfo cacheInfo)
        {
            caches[GetKey(cacheInfo.TenantId, cacheInfo.Resource, cacheInfo.LoginType)] = cacheInfo;
        }

        public void Clone(AuthTokenCache tokenCache)
        {
            caches = tokenCache.caches;
        }

        private string GetKey(string tenantId, string resource, LoginType loginType)
        {
            return string.Format("{0}::{1}::{2}", tenantId, resource, loginType.ToString());
        }
    }
}
