﻿using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure.DataModel.Authorization;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure.DataModel.Compute;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure.DataModel.Network;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure.DataModel.ResourceManager;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure.DataModel.SQLDatabase;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure.DataModel.Storage;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.Samples.CloudSolutionProviderManagement.Azure
{
    /// <summary>
    /// 
    /// References:
    /// Resource Management API: https://msdn.microsoft.com/en-us/library/azure/dn790568.aspx
    /// Compute API: https://msdn.microsoft.com/en-us/library/azure/mt163647.aspx
    /// Network API: https://msdn.microsoft.com/en-us/library/azure/mt163658.aspx
    /// Storage API: https://msdn.microsoft.com/en-us/library/azure/mt163683.aspx
    /// SQL Database API: https://msdn.microsoft.com/en-us/library/azure/mt163571.aspx
    /// </summary>
    public static class AzureResourceManager
    {
        #region Resource Management API
        public static async Task<string> DeleteResourceGroup(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string name)
        {
            string operationStatusLink=null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourcegroups/{2}?api-version={3}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, name, Constants.AzureResourceManagerAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureResourceManagementApiInvoke(uri, cacheInfo, "Delete", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        operationStatusLink = response.Headers.GetValues("Location").FirstOrDefault<string>();
                        
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return operationStatusLink;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="name"></param>
        /// <param name="parameters"></param>
        /// <returns></returns>
        public static async Task<ResourceGroup> CreateResourceGroup(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string name, CreateResourceGroupParameters parameters)
        {
            ResourceGroup resourceGroup = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourcegroups/{2}?api-version={3}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, name, Constants.AzureResourceManagerAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(parameters), Encoding.UTF8, Constants.JsonContentType); 
                using (HttpResponseMessage response = await AzureResourceManagementApiInvoke(uri, cacheInfo, "Put", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        resourceGroup = JsonConvert.DeserializeObject<ResourceGroup>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return resourceGroup;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="name"></param>
        /// <param name="parameters"></param>
        /// <returns></returns>
        public static async Task<ResourceGroup> UpdateResourceGroup(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string name, CreateResourceGroupParameters parameters)
        {
            ResourceGroup resourceGroup = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourcegroups/{2}?api-version={3}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, name, Constants.AzureResourceManagerAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(parameters), Encoding.UTF8, Constants.JsonContentType); ;
                using (HttpResponseMessage response = await AzureResourceManagementApiInvoke(uri, cacheInfo, "Patch", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        resourceGroup = JsonConvert.DeserializeObject<ResourceGroup>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return resourceGroup;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="name"></param>
        /// <returns></returns>
        public static async Task<ResourceGroup> GetResourceGroupByName(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string name)
        {
            ResourceGroup resourceGroup = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourcegroups/{2}?api-version={3}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, name, Constants.AzureResourceManagerAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureResourceManagementApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        resourceGroup = JsonConvert.DeserializeObject<ResourceGroup>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return resourceGroup;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="queryString"></param>
        /// <returns></returns>
        public static async Task<ResourceGroupList> GetResourceGroupList(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string queryString)
        {
            ResourceGroupList resourceGroupList = null;
            try
            {
                if (!string.IsNullOrEmpty(queryString))
                    queryString = "&" + queryString;
                else
                    queryString = string.Empty;
                var path = string.Format("{0}/subscriptions/{1}/resourcegroups?api-version={2}{3}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, Constants.AzureResourceManagerAPIVersion, queryString);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureResourceManagementApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        resourceGroupList = JsonConvert.DeserializeObject<ResourceGroupList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return resourceGroupList;
        }

        public static async Task<LocationList> GetLocationList(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId)
        {
            LocationList locationList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/locations?api-version={2}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, Constants.AzureResourceManagerAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureResourceManagementApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        locationList = JsonConvert.DeserializeObject<LocationList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return locationList;
        }

        public static async Task<string> CreateUpdateDeployment(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string deploymentName, CreateUpdateDeploymentParameters parameters)
        {
            string operationStatusLink = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourcegroups/{2}/deployments/{3}?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, deploymentName, Constants.AzureResourceManagerAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(parameters), Encoding.UTF8, Constants.JsonContentType);
                using (HttpResponseMessage response = await AzureResourceManagementApiInvoke(uri, cacheInfo, "Put", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        if (response.Headers.Contains("Location"))
                        {
                            operationStatusLink = response.Headers.GetValues("Location").FirstOrDefault<string>();
                        }
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return operationStatusLink;
        }

        public static async Task<string> DeleteDeployment(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string deploymentName)
        {
            string operationStatusLink = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourcegroups/{2}/deployments/{3}?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, deploymentName, Constants.AzureResourceManagerAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureResourceManagementApiInvoke(uri, cacheInfo, "Delete", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        if (response.Headers.Contains("Location"))
                        {
                            operationStatusLink = response.Headers.GetValues("Location").FirstOrDefault<string>();
                        }
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return operationStatusLink;
        }

        public static async Task<DeploymentList> GetDeploymentList(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName)
        {
            DeploymentList deploymentList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourcegroups/{2}/deployments?api-version={3}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, Constants.AzureResourceManagerAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureResourceManagementApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        deploymentList = JsonConvert.DeserializeObject<DeploymentList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return deploymentList;
        }

        public static async Task<AzureSubscriptionList> GetAzureSubscriptionList(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler)
        {
            AzureSubscriptionList subscriptionList = null;
            try
            {
                var path = string.Format("{0}/subscriptions?api-version={2}", Constants.AzureResourceManagerAPIsUrl, Constants.AzureResourceManagerAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureResourceManagementApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        subscriptionList = JsonConvert.DeserializeObject<AzureSubscriptionList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return subscriptionList;
        }

        public static async Task<AzureSubscription> GetAzureSubscription(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId)
        {
            AzureSubscription subscription = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}?api-version={2}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, Constants.AzureResourceManagerAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureResourceManagementApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        subscription = JsonConvert.DeserializeObject<AzureSubscription>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return subscription;
        }
        #endregion

        #region Authorization
        public static async Task<RoleDefinitionList> GetRoleDefinitionList(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId)
        {
            RoleDefinitionList rdList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/providers/Microsoft.Authorization/roleDefinitions?api-version={2}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, Constants.AzureComputeAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureComputeApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        rdList = JsonConvert.DeserializeObject<RoleDefinitionList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return rdList;
        }

        public static async Task<RoleAssignment> CreateRoleAssignment(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string scope, string assignmentName, CreateRoleAssignmentParameters parameters)
        {
            RoleAssignment assignment = null;
            try
            {
                var path = string.Format("{0}/{1}/providers/Microsoft.Authorization/roleAssignments/{2}?api-version={3}", Constants.AzureResourceManagerAPIsUrl, scope, assignmentName, Constants.AzureComputeAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureComputeApiInvoke(uri, cacheInfo, "Put", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        assignment = JsonConvert.DeserializeObject<RoleAssignment>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return assignment;
        }
        #endregion

        #region Compute
        public static async Task<VirtualMachineList> GetVirtualMachineList(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId)
        {
            VirtualMachineList vmList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/providers/Microsoft.Compute/virtualMachines?api-version={2}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, Constants.AzureComputeAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureComputeApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        vmList = JsonConvert.DeserializeObject<VirtualMachineList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return vmList;
        }

        public static async Task<VirtualMachineList> GetVirtualMachineListByResourceGroup(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName)
        {
            VirtualMachineList vmList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Compute/virtualMachines?api-version={3}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, Constants.AzureComputeAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureComputeApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        vmList = JsonConvert.DeserializeObject<VirtualMachineList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return vmList;
        }

        public static async Task<string> StartVirtualMachine(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string vmName)
        {
            string operationStatusLink = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Compute/virtualMachines/{3}/start?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, vmName, Constants.AzureComputeAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureComputeApiInvoke(uri, cacheInfo, "Post", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        if (response.Headers.Contains("Azure-AsyncOperation"))
                        {
                            operationStatusLink = response.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault<string>();
                        }
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return operationStatusLink;
        }

        public static async Task<string> RestartVirtualMachine(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string vmName)
        {
            string operationStatusLink = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Compute/virtualMachines/{3}/restart?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, vmName, Constants.AzureComputeAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureComputeApiInvoke(uri, cacheInfo, "Post", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        if (response.Headers.Contains("Azure-AsyncOperation"))
                        {
                            operationStatusLink = response.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault<string>();
                        }
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return operationStatusLink;
        }

        public static async Task<string> StopVirtualMachine(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string vmName)
        {
            string operationStatusLink = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Compute/virtualMachines/{3}/powerOff?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, vmName, Constants.AzureComputeAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureComputeApiInvoke(uri, cacheInfo, "Post", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        if (response.Headers.Contains("Azure-AsyncOperation"))
                        {
                            operationStatusLink = response.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault<string>();
                        }
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return operationStatusLink;
        }

        public static async Task<string> DeleteVirtualMachine(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string vmName)
        {
            string operationStatusLink = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Compute/virtualMachines/{3}?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, vmName, Constants.AzureComputeAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureComputeApiInvoke(uri, cacheInfo, "Delete", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        if (response.Headers.Contains("Azure-AsyncOperation"))
                        {
                            operationStatusLink = response.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault<string>();
                        }
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return operationStatusLink;
        }

        public static async Task<string> DeallocateVirtualMachine(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string vmName)
        {
            string operationStatusLink = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Compute/virtualMachines/{3}/deallocate?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, vmName, Constants.AzureComputeAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureComputeApiInvoke(uri, cacheInfo, "Post", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        if (response.Headers.Contains("Azure-AsyncOperation"))
                        {
                            operationStatusLink = response.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault<string>();
                        }
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return operationStatusLink;
        }

        public static async Task<string> CaptureVirtualMachine(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string vmName)
        {
            string operationStatusLink = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Compute/virtualMachines/{3}/capture?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, vmName, Constants.AzureComputeAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureComputeApiInvoke(uri, cacheInfo, "Post", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        if (response.Headers.Contains("Azure-AsyncOperation"))
                        {
                            operationStatusLink = response.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault<string>();
                        }
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return operationStatusLink;
        }

        public static async Task<bool> GeneralizeVirtualMachine(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string vmName)
        {
            bool success=false;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Compute/virtualMachines/{3}/generalize?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, vmName, Constants.AzureComputeAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureComputeApiInvoke(uri, cacheInfo, "Post", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        success = true;
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return success;
        }

        public static async Task<AvailabilitySetList> GetAvailabilitySetList(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId)
        {
            AvailabilitySetList availabilitySetList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/providers/Microsoft.Compute/availabilitySets?api-version={2}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, Constants.AzureComputeAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureComputeApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        availabilitySetList = JsonConvert.DeserializeObject<AvailabilitySetList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return availabilitySetList;
        }

        public static async Task<AvailabilitySetList> GetAvailabilitySetListByResourceGroup(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName)
        {
            AvailabilitySetList availabilitySetList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Compute/availabilitySets?api-version={3}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, Constants.AzureComputeAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureComputeApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        availabilitySetList = JsonConvert.DeserializeObject<AvailabilitySetList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return availabilitySetList;
        }
        #endregion

        #region Network
        public static async Task<NetworkInterfaceList> GetNetworkInterfaceList(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId)
        {
            NetworkInterfaceList networkInterfaceList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/providers/Microsoft.Network/networkInterfaces?api-version={2}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, Constants.AzureNetworkAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureNetworkApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        networkInterfaceList = JsonConvert.DeserializeObject<NetworkInterfaceList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return networkInterfaceList;
        }

        public static async Task<NetworkInterfaceList> GetNetworkInterfaceListByResourceGroup(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName)
        {
            NetworkInterfaceList networkInterfaceList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Network/networkInterfaces?api-version={3}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, Constants.AzureNetworkAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureNetworkApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        networkInterfaceList = JsonConvert.DeserializeObject<NetworkInterfaceList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return networkInterfaceList;
        }

        public static async Task<LoadBalancerList> GetLoadBalancerList(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId)
        {
            LoadBalancerList loadBalancerList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/providers/Microsoft.Network/loadBalancers?api-version={2}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, Constants.AzureNetworkAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureNetworkApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        loadBalancerList = JsonConvert.DeserializeObject<LoadBalancerList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return loadBalancerList;
        }

        public static async Task<LoadBalancerList> GetLoadBalancerListByResourceGroup(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName)
        {
            LoadBalancerList loadBalancerList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/(2)/providers/Microsoft.Network/loadBalancers?api-version={3}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, Constants.AzureNetworkAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureNetworkApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        loadBalancerList = JsonConvert.DeserializeObject<LoadBalancerList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return loadBalancerList;
        }

        public static async Task<VirtualNetworkList> GetVirtualNetworkList(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId)
        {
            VirtualNetworkList virtualNetworkList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/providers/Microsoft.Network/virtualNetworks?api-version={2}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, Constants.AzureNetworkAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureNetworkApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        virtualNetworkList = JsonConvert.DeserializeObject<VirtualNetworkList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return virtualNetworkList;
        }

        public static async Task<VirtualNetworkList> GetVirtualNetworkListByResourceGroup(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName)
        {
            VirtualNetworkList virtualNetworkList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Network/virtualNetworks?api-version={3}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, Constants.AzureNetworkAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureNetworkApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        virtualNetworkList = JsonConvert.DeserializeObject<VirtualNetworkList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return virtualNetworkList;
        }

        public static async Task<string> CreateUpdateNetworkInterface(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string networkInterfaceName, NetworkInterface parameters)
        {
            string operationStatusLink = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Network/networkInterfaces/{3}?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, networkInterfaceName, Constants.AzureNetworkAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(parameters), Encoding.UTF8, Constants.JsonContentType); ;
                using (HttpResponseMessage response = await AzureNetworkApiInvoke(uri, cacheInfo, "Put", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        if (response.Headers.Contains("Azure-AsyncOperation"))
                        {
                            operationStatusLink = response.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault<string>();
                        }
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return operationStatusLink;
        }

        public static async Task<string> DeleteNetworkInterface(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string networkInterfaceName)
        {
            string operationStatusLink = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Network/networkInterfaces/{3}?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, networkInterfaceName, Constants.AzureNetworkAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureNetworkApiInvoke(uri, cacheInfo, "Delete", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        if (response.Headers.Contains("Azure-AsyncOperation"))
                        {
                            operationStatusLink = response.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault<string>();
                        }
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return operationStatusLink;
        }

        public static async Task<string> CreateUpdateVirtualNetwork(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string virtualNetworkName, VirtualNetwork parameters)
        {
            string operationStatusLink = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Network/virtualNetworks/{3}?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, virtualNetworkName, Constants.AzureNetworkAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(parameters), Encoding.UTF8, Constants.JsonContentType); ;
                using (HttpResponseMessage response = await AzureNetworkApiInvoke(uri, cacheInfo, "Put", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        if (response.Headers.Contains("Azure-AsyncOperation"))
                        {
                            operationStatusLink = response.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault<string>();
                        }
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return operationStatusLink;
        }

        public static async Task<string> DeleteVirtualNetwork(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string virtualNetworkName)
        {
            string operationStatusLink = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Network/virtualNetworks/{3}?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, virtualNetworkName, Constants.AzureNetworkAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureNetworkApiInvoke(uri, cacheInfo, "Delete", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        if (response.Headers.Contains("Azure-AsyncOperation"))
                        {
                            operationStatusLink = response.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault<string>();
                        }
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return operationStatusLink;
        }

        public static async Task<string> CreateUpdateLoadBalancer(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string loadBalancerName, LoadBalancer parameters)
        {
            string operationStatusLink = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Network/loadBalancers/{3}?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, loadBalancerName, Constants.AzureNetworkAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(parameters), Encoding.UTF8, Constants.JsonContentType); ;
                using (HttpResponseMessage response = await AzureNetworkApiInvoke(uri, cacheInfo, "Put", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        if (response.Headers.Contains("Azure-AsyncOperation"))
                        {
                            operationStatusLink = response.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault<string>();
                        }
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return operationStatusLink;
        }

        public static async Task<string> DeleteLoadBalancer(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string loadBalancerName)
        {
            string operationStatusLink = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Network/virtualNetworks/{3}?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, loadBalancerName, Constants.AzureNetworkAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureNetworkApiInvoke(uri, cacheInfo, "Delete", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        if (response.Headers.Contains("Azure-AsyncOperation"))
                        {
                            operationStatusLink = response.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault<string>();
                        }
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return operationStatusLink;
        }
        #endregion

        #region Storage
        public static async Task<string> CreateStorageAccount(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string storageAccountName, CreateStorageAccountParameters parameters)
        {
            string operationStatusLink = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourcegroups/{2}/providers/Microsoft.Storage/storageAccounts/{3}?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, storageAccountName, Constants.AzureStorageAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(parameters), Encoding.UTF8, Constants.JsonContentType); ;
                using (HttpResponseMessage response = await AzureStorageApiInvoke(uri, cacheInfo, "Put", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        if (response.Headers.Contains("Location"))
                        {
                            operationStatusLink = response.Headers.GetValues("Location").FirstOrDefault<string>();
                        }
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return operationStatusLink;
        }

        public static async Task<bool> DeleteStorageAccount(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string accountName)
        {
            bool success=false;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourcegroups/{2}/providers/Microsoft.Storage/storageAccounts/{3}?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, accountName, Constants.AzureStorageAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureStorageApiInvoke(uri, cacheInfo, "Delete", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        success = true;
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return success;
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <param name="subscriptionId"></param>
        /// <returns></returns>
        public static async Task<StorageAccountList> GetStorageAccountList(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId)
        {
            StorageAccountList storageAccountList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/providers/Microsoft.Storage/storageAccounts?api-version={2}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, Constants.AzureStorageAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureStorageApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        storageAccountList = JsonConvert.DeserializeObject<StorageAccountList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return storageAccountList;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="resourceGroupName"></param>
        /// <returns></returns>
        public static async Task<StorageAccountList> GetStorageAccountListByResourceGroup(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName)
        {
            StorageAccountList storageAccountList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Storage/storageAccounts?api-version={3}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, Constants.AzureStorageAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureStorageApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        storageAccountList = JsonConvert.DeserializeObject<StorageAccountList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return storageAccountList;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="resourceGroupName"></param>
        /// <param name="storageAccountName"></param>
        /// <returns></returns>
        public static async Task<StorageAccountKeys> ListStorageAccountKeys(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string storageAccountName)
        {
            StorageAccountKeys storageAccountKeys = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Storage/storageAccounts/{3}/listKeys?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, storageAccountName, Constants.AzureStorageAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureStorageApiInvoke(uri, cacheInfo, "Post", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        storageAccountKeys = JsonConvert.DeserializeObject<StorageAccountKeys>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return storageAccountKeys;
        }
        #endregion

        #region Azure SQL Database
        /// <summary>
        /// 
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="resourceGroupName"></param>
        /// <param name="serverName"></param>
        /// <param name="databaseName"></param>
        /// <param name="parameters"></param>
        /// <returns></returns>
        public static async Task<Database> CreateUpdateAzureSqlDatabase(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string serverName, string databaseName, DatabaseCreateUpdateParameters parameters)
        {
            Database database = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Sql/servers/{3}/databases/{4}?api-version={5}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, serverName, databaseName, Constants.AzureSQLDatabaseAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(parameters), Encoding.UTF8, Constants.JsonContentType); 
                using (HttpResponseMessage response = await AzureSqlApiInvoke(uri, cacheInfo, "Put", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        database = JsonConvert.DeserializeObject<Database>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return database;
        }

        public static async Task<bool> DeleteAzureSqlDatabase(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string serverName, string databaseName)
        {
            bool isSuccess = false;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Sql/servers/{3}/databases/{4}?api-version={5}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, serverName, databaseName, Constants.AzureSQLDatabaseAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null ;
                using (HttpResponseMessage response = await AzureSqlApiInvoke(uri, cacheInfo, "Delete", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        isSuccess = true;
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return isSuccess;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="resourceGroupName"></param>
        /// <param name="serverName"></param>
        /// <returns></returns>
        public static async Task<DatabaseList> ListAzureSqlDatabasesByServer(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string serverName)
        {
            DatabaseList databaseList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Sql/servers/{3}/databases?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, serverName, Constants.AzureSQLDatabaseAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureSqlApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        databaseList = JsonConvert.DeserializeObject<DatabaseList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return databaseList;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="resourceGroupName"></param>
        /// <param name="serverName"></param>
        /// <param name="elasticPoolName"></param>
        /// <returns></returns>
        public static async Task<DatabaseList> ListAzureSqlDatabasesByElasticPool(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string serverName, string elasticPoolName)
        {
            DatabaseList databaseList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Sql/servers/{3}/elasticPools/{4}/databases?api-version={5}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, serverName, elasticPoolName, Constants.AzureSQLDatabaseAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureSqlApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        databaseList = JsonConvert.DeserializeObject<DatabaseList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return databaseList;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="resourceGroupName"></param>
        /// <param name="serverName"></param>
        /// <param name="elasticPoolName"></param>
        /// <param name="parameters"></param>
        /// <returns></returns>
        public static async Task<ElasticPool> CreateUpdateAzureSqlElasticPool(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string serverName, string elasticPoolName, ElasticPoolCreateUpdateParameters parameters)
        {
            ElasticPool elasticPool = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Sql/servers/{3}/elasticPools/{4}?api-version={5}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, serverName, elasticPoolName, Constants.AzureSQLDatabaseAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(parameters), Encoding.UTF8, Constants.JsonContentType); 
                using (HttpResponseMessage response = await AzureSqlApiInvoke(uri, cacheInfo, "Put", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        elasticPool = JsonConvert.DeserializeObject<ElasticPool>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return elasticPool;
        }

        public static async Task<bool> DeleteAzureSqlElasticPool(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string serverName, string elasticPoolName)
        {
            bool isSuccess = false;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Sql/servers/{3}/elasticPools/{4}?api-version={5}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, serverName, elasticPoolName, Constants.AzureSQLDatabaseAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureSqlApiInvoke(uri, cacheInfo, "Delete", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        isSuccess = true;
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return isSuccess;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="resourceGroupName"></param>
        /// <param name="serverName"></param>
        /// <returns></returns>
        public static async Task<ElasticPoolList> ListAzureSqlElasticPoolsByServer(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string serverName)
        {
            ElasticPoolList elasticPoolList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Sql/servers/{3}/elasticPools?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, serverName, Constants.AzureSQLDatabaseAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureSqlApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        elasticPoolList = JsonConvert.DeserializeObject<ElasticPoolList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return elasticPoolList;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="resourceGroupName"></param>
        /// <param name="serverName"></param>
        /// <param name="parameters"></param>
        /// <returns></returns>
        public static async Task<Server> CreateUpdateAzureSqlServer(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string serverName, ServerCreateUpdateParameters parameters)
        {
            Server server = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Sql/servers/{3}?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, serverName, Constants.AzureSQLDatabaseAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(parameters), Encoding.UTF8, Constants.JsonContentType); ;
                using (HttpResponseMessage response = await AzureSqlApiInvoke(uri, cacheInfo, "Put", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        server = JsonConvert.DeserializeObject<Server>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return server;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="resourceGroupName"></param>
        /// <param name="serverName"></param>
        /// <returns></returns>
        public static async Task<bool> DeleteAzureSqlServer(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string serverName)
        {
            bool isSuccess = false;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Sql/servers/{3}?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, serverName, Constants.AzureSQLDatabaseAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureSqlApiInvoke(uri, cacheInfo, "Delete", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        isSuccess = true;
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return isSuccess;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="resourceGroupName"></param>
        /// <returns></returns>
        public static async Task<ServerList> ListAzureSqlServers(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName)
        {
            ServerList serverList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Sql/servers?api-version={3}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, Constants.AzureSQLDatabaseAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureSqlApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        serverList = JsonConvert.DeserializeObject<ServerList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return serverList;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="resourceGroupName"></param>
        /// <param name="serverName"></param>
        /// <returns></returns>
        public static async Task<ServiceObjectiveList> ListAzureSqlServiceObjectives(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string serverName)
        {
            ServiceObjectiveList serviceObjectiveList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Sql/servers/{3}/serviceObjectives?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, serverName, Constants.AzureSQLDatabaseAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureSqlApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        serviceObjectiveList = JsonConvert.DeserializeObject<ServiceObjectiveList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return serviceObjectiveList;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="resourceGroupName"></param>
        /// <param name="serverName"></param>
        /// <returns></returns>
        public static async Task<UsageMetricList> ListAzureSqlUsageMetricsByServer(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string serverName)
        {
            UsageMetricList usageMetricList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Sql/servers/{3}/usages?api-version={4}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, serverName, Constants.AzureSQLDatabaseAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureSqlApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        usageMetricList = JsonConvert.DeserializeObject<UsageMetricList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return usageMetricList;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <param name="subscriptionId"></param>
        /// <param name="resourceGroupName"></param>
        /// <param name="serverName"></param>
        /// <param name="databaseName"></param>
        /// <returns></returns>
        public static async Task<UsageMetricList> ListAzureSqlUsageMetricsByDatabase(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string subscriptionId, string resourceGroupName, string serverName, string databaseName)
        {
            UsageMetricList usageMetricList = null;
            try
            {
                var path = string.Format("{0}/subscriptions/{1}/resourceGroups/{2}/providers/Microsoft.Sql/servers/{3}/databases/{4}/usages?api-version={5}", Constants.AzureResourceManagerAPIsUrl, subscriptionId, resourceGroupName, serverName, databaseName, Constants.AzureSQLDatabaseAPIVersion);
                Uri uri = new Uri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await AzureSqlApiInvoke(uri, cacheInfo, "Get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        usageMetricList = JsonConvert.DeserializeObject<UsageMetricList>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return usageMetricList;
        }
        #endregion

        /// <summary>
        /// 
        /// </summary>
        /// <param name="uri"></param>
        /// <param name="cacheInfo"></param>
        /// <param name="verb"></param>
        /// <param name="handler"></param>
        /// <param name="content"></param>
        /// <returns></returns>
        public static async Task<HttpResponseMessage> AzureResourceManagementApiInvoke(Uri uri, AuthTokenCacheInfo cacheInfo, string verb, DelegatingHandler handler, HttpContent content)
        {
            using (var client = new HttpClient(handler))
            {
                HttpResponseMessage response = null;
                client.DefaultRequestHeaders.Add("Authorization", cacheInfo.GetAuthorizationHeader());
                client.DefaultRequestHeaders.Add("Accept", Constants.JsonContentType);
                client.DefaultRequestHeaders.Add("x-ms-request-id", Guid.NewGuid().ToString());
                if (String.Equals(verb, "get", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.GetAsync(uri);
                }
                else if (String.Equals(verb, "delete", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.DeleteAsync(uri);
                }
                else if (String.Equals(verb, "post", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.PostAsync(uri, content);
                }
                else if (String.Equals(verb, "put", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.PutAsync(uri, content);
                }
                else if (String.Equals(verb, "patch", StringComparison.OrdinalIgnoreCase))
                {
                    using (var message = new HttpRequestMessage(new HttpMethod("PATCH"), uri))
                    {
                        message.Content = content;
                        response = await client.SendAsync(message).ConfigureAwait(false);
                    }
                }
                else
                {
                    throw new InvalidOperationException(String.Format("Invalid http verb {0}!", verb));
                }
                return response;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="uri"></param>
        /// <param name="cacheInfo"></param>
        /// <param name="verb"></param>
        /// <param name="handler"></param>
        /// <param name="content"></param>
        /// <returns></returns>
        public static async Task<HttpResponseMessage> AzureSqlApiInvoke(Uri uri, AuthTokenCacheInfo cacheInfo, string verb, DelegatingHandler handler, HttpContent content)
        {
            using (var client = new HttpClient(handler))
            {
                HttpResponseMessage response = null;
                client.DefaultRequestHeaders.Add("Authorization", cacheInfo.GetAuthorizationHeader());
                client.DefaultRequestHeaders.Add("Accept", Constants.JsonContentType);
                client.DefaultRequestHeaders.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                if (String.Equals(verb, "get", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.GetAsync(uri);
                }
                else if (String.Equals(verb, "delete", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.DeleteAsync(uri);
                }
                else if (String.Equals(verb, "post", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.PostAsync(uri, content);
                }
                else if (String.Equals(verb, "put", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.PutAsync(uri, content);
                }
                else if (String.Equals(verb, "patch", StringComparison.OrdinalIgnoreCase))
                {
                    using (var message = new HttpRequestMessage(new HttpMethod("PATCH"), uri))
                    {
                        message.Content = content;
                        response = await client.SendAsync(message).ConfigureAwait(false);
                    }
                }
                else
                {
                    throw new InvalidOperationException(String.Format("Invalid http verb {0}!", verb));
                }
                return response;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="uri"></param>
        /// <param name="cacheInfo"></param>
        /// <param name="verb"></param>
        /// <param name="handler"></param>
        /// <param name="content"></param>
        /// <returns></returns>
        public static async Task<HttpResponseMessage> AzureNetworkApiInvoke(Uri uri, AuthTokenCacheInfo cacheInfo, string verb, DelegatingHandler handler, HttpContent content)
        {
            using (var client = new HttpClient(handler))
            {
                HttpResponseMessage response = null;
                client.DefaultRequestHeaders.Add("Authorization", cacheInfo.GetAuthorizationHeader());
                client.DefaultRequestHeaders.Add("Accept", Constants.JsonContentType);
                if (String.Equals(verb, "get", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.GetAsync(uri);
                }
                else if (String.Equals(verb, "delete", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.DeleteAsync(uri);
                }
                else if (String.Equals(verb, "post", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.PostAsync(uri, content);
                }
                else if (String.Equals(verb, "put", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.PutAsync(uri, content);
                }
                else if (String.Equals(verb, "patch", StringComparison.OrdinalIgnoreCase))
                {
                    using (var message = new HttpRequestMessage(new HttpMethod("PATCH"), uri))
                    {
                        message.Content = content;
                        response = await client.SendAsync(message).ConfigureAwait(false);
                    }
                }
                else
                {
                    throw new InvalidOperationException(String.Format("Invalid http verb {0}!", verb));
                }
                return response;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="uri"></param>
        /// <param name="cacheInfo"></param>
        /// <param name="verb"></param>
        /// <param name="handler"></param>
        /// <param name="content"></param>
        /// <returns></returns>
        public static async Task<HttpResponseMessage> AzureComputeApiInvoke(Uri uri, AuthTokenCacheInfo cacheInfo, string verb, DelegatingHandler handler, HttpContent content)
        {
            using (var client = new HttpClient(handler))
            {
                HttpResponseMessage response = null;
                client.DefaultRequestHeaders.Add("Authorization", cacheInfo.GetAuthorizationHeader());
                client.DefaultRequestHeaders.Add("Accept", Constants.JsonContentType);
                client.DefaultRequestHeaders.Add("x-ms-request-id", Guid.NewGuid().ToString());
                if (String.Equals(verb, "get", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.GetAsync(uri);
                }
                else if (String.Equals(verb, "delete", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.DeleteAsync(uri);
                }
                else if (String.Equals(verb, "post", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.PostAsync(uri, content);
                }
                else if (String.Equals(verb, "put", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.PutAsync(uri, content);
                }
                else if (String.Equals(verb, "patch", StringComparison.OrdinalIgnoreCase))
                {
                    using (var message = new HttpRequestMessage(new HttpMethod("PATCH"), uri))
                    {
                        message.Content = content;
                        response = await client.SendAsync(message).ConfigureAwait(false);
                    }
                }
                else
                {
                    throw new InvalidOperationException(String.Format("Invalid http verb {0}!", verb));
                }
                return response;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="uri"></param>
        /// <param name="cacheInfo"></param>
        /// <param name="verb"></param>
        /// <param name="handler"></param>
        /// <param name="content"></param>
        /// <returns></returns>
        public static async Task<HttpResponseMessage> AzureStorageApiInvoke(Uri uri, AuthTokenCacheInfo cacheInfo, string verb, DelegatingHandler handler, HttpContent content)
        {
            using (var client = new HttpClient(handler))
            {
                HttpResponseMessage response = null;
                client.DefaultRequestHeaders.Add("Authorization", cacheInfo.GetAuthorizationHeader());
                client.DefaultRequestHeaders.Add("Accept", Constants.JsonContentType);
                client.DefaultRequestHeaders.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                if (String.Equals(verb, "get", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.GetAsync(uri);
                }
                else if (String.Equals(verb, "delete", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.DeleteAsync(uri);
                }
                else if (String.Equals(verb, "post", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.PostAsync(uri, content);
                }
                else if (String.Equals(verb, "put", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.PutAsync(uri, content);
                }
                else if (String.Equals(verb, "patch", StringComparison.OrdinalIgnoreCase))
                {
                    using (var message = new HttpRequestMessage(new HttpMethod("PATCH"), uri))
                    {
                        message.Content = content;
                        response = await client.SendAsync(message).ConfigureAwait(false);
                    }
                }
                else
                {
                    throw new InvalidOperationException(String.Format("Invalid http verb {0}!", verb));
                }
                return response;
            }
        }
    }
}
