﻿using Microsoft.Samples.CloudSolutionProviderManagement.Azure.DataModel.ResourceManager;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.Samples.CloudSolutionProviderManagement.Azure.DataModel.Compute
{
    public class VirtualMachineList
    {
        [JsonProperty("value")]
        public VirtualMachine[] VirtualMachines { get; set; }
    }

    public class VirtualMachine
    {
        [JsonProperty("id")]
        public string Id { get; set; }
        [JsonProperty("name")]
        public string Name { get; set; }
        [JsonProperty("type")]
        public string Type { get; set; }
        [JsonProperty("location")]
        public string Location { get; set; }
        [JsonProperty("tags")]
        public IDictionary<string, string> Tags { get; set; }
        [JsonProperty("plan")]
        public Plan Plan { get; set; }
        [JsonProperty("properties")]
        public VirtualMachineProperties Properties { get; set; }
    }

    public class VirtualMachineProperties
    {
        [JsonProperty("availabilitySet")]
        public AvailabilitySetReference AvailabilitySet { get; set; }
        [JsonProperty("hardwareProfile")]
        public HardwareProfile HardwareProfile { get; set; }
        [JsonProperty("storageProfile")]
        public StorageProfile StorageProfile { get; set; }
        [JsonProperty("osProfile")]
        public OSProfile OSProfile { get; set; }
        [JsonProperty("networkProfile")]
        public NetworkProfile NetworkProfile { get; set; }
        [JsonProperty("provisioningState")]
        public string ProvisioningState { get; set; }
        [JsonProperty("resources")]
        public VirtualMachineExtension[] Extensions { get; set; }
        [JsonProperty("instaceView")]
        public VirtualMachineInstanceView InstanceView { get; set; }
    }

    public class VirtualMachineInstanceView
    {
        [JsonProperty("platformUpdateDomain")]
        public int? PlatformUpdateDomain { get; set; }
        [JsonProperty("platformFaultDomain")]
        public int? PlatformFaultDomain { get; set; }
        [JsonProperty("rdpThumbPrint")]
        public string RdpThumbPrint { get; set;}
        [JsonProperty("vmAgent")]
        public VirtualMachineAgentInstanceView VMAgent { get; set; }
        [JsonProperty("disks")]
        public DiskInstanceView[] Disks { get; set; }
        [JsonProperty("extensions")]
        public VirtualMachineExtensionInstanceView[] Extensions { get; set; }
        [JsonProperty("statuses")]
        public InstanceViewStatus[] Statuses { get; set; }
    }

    public class DiskInstanceView
    {
        [JsonProperty("name")]
        public string Name { get; set; }
        [JsonProperty("statuses")]
        public InstanceViewStatus[] Statuses { get; set; }
    }

    public class VirtualMachineAgentInstanceView
    {
        [JsonProperty("statuses")]
        public InstanceViewStatus[] Statsuses { get; set; }
        [JsonProperty("extensionHandlers")]
        public VirtualMachineExtensionHandlerInstanceView ExtensionHandlers { get; set; }
        [JsonProperty("vmAgentversion")]
        public string VMAgentVersion { get; set; }
    }

    public class VirtualMachineExtensionHandlerInstanceView
    {
        [JsonProperty("type")]
        public string Type { get; set; }
        [JsonProperty("typeHandlerVersion")]
        public string TypeHandlerVersion { get; set; }
        [JsonProperty("status")]
        public InstanceViewStatus Status { get; set; }
    }

    public class HardwareProfile
    {
        [JsonProperty("vmSize")]
        public string VirtualMachineSize { get; set; }
    }

    public class StorageProfile
    {
        [JsonProperty("imageReference")]
        public ImageReference ImageReference { get; set; }
        [JsonProperty("osDisk")]
        public OSDisk OSDisk { get; set; }
        [JsonProperty("dataDisks")]
        public DataDisk[] DataDisks { get; set; }
    }

    public class OSProfile
    {
        [JsonProperty("computerName")]
        public string ComputerName { get; set; }
        [JsonProperty("adminPassword")]
        public string AdminPassword { get; set; }
        [JsonProperty("adminUsername")]
        public string AdminUsername { get; set; }
        [JsonProperty("customData")]
        public string CustomData { get; set; }
        [JsonProperty("windowsConfiguration")]
        public WindowsConfiguration WindowsConfiguration { get; set; }
        [JsonProperty("linuxConfiguration")]
        public LinuxConfiguration LinuxConfiguration { get; set; }
        [JsonProperty("secrets")]
        public VaultSecret[] Secrets { get; set; }
    }

    public class NetworkProfile
    {
        [JsonProperty("networkInterfaces")]
        public NetworkInterfaceReference[] NetworkInterfaceReferences { get; set; }
    }

    public class VaultSecret
    {
        [JsonProperty("sourceVault")]
        public SourceVaultReference SourcevaultReference { get; set; }
        [JsonProperty("vaultCertificates")]
        public VaultCetificate[] VaultCertificates { get; set; }
    }

    public class SourceVaultReference
    {
        [JsonProperty("id")]
        public string Id { get; set; }
    }

    public class VaultCetificate
    {
        [JsonProperty("certificateUrl")]
        public string CertificateUrl { get; set; }
        [JsonProperty("certificateStore")]
        public string certificateStore { get; set; }
    }
    public class ImageReference
    {
        [JsonProperty("offer")]
        public string Offer { get; set; }
        [JsonProperty("publisher")]
        public string Publisher { get; set; }
        [JsonProperty("sku")]
        public string Sku { get; set; }
        [JsonProperty("version")]
        public string Version { get; set; }
    }

    public class NetworkInterfaceReference
    {
        [JsonProperty("id")]
        public string Id { get; set; }
        [JsonProperty("properties")]
        public NetworkInterfaceReferenceProperties Properties { get; set; }
    }

    public class AvailabilitySetReference
    {
        [JsonProperty("id")]
        public string Id { get; set; }
    }

    public class NetworkInterfaceReferenceProperties
    {
        [JsonProperty("primary")]
        public bool? Primary { get; set; }
    }

    public class LinuxConfiguration
    {
        [JsonProperty("disablePasswordAuthentication")]
        public bool? DisablePasswordAuthentication { get; set; }
        [JsonProperty("ssh")]
        public SshConfiguration SshConfiguration { get; set; }
    }

    public class SshConfiguration
    {
        [JsonProperty("publicKeys")]
        public SshPublicKey[] PublicKeys { get; set; }
    }

    public class SshPublicKey
    {
        [JsonProperty("path")]
        public string Path { get; set; }
        [JsonProperty("keyData")]
        public string KeyData { get; set; }
    }
    public class DataDisk
    {
        [JsonProperty("caching")]
        public string caching { get; set; }
        [JsonProperty("createOption")]
        public string CreateOption { get; set; }
        [JsonProperty("name")]
        public string Name { get; set; }
        [JsonProperty("image")]
        public VirtualHardDisk SourceImage { get; set; }
        [JsonProperty("vhd")]
        public VirtualHardDisk VirtualHardDisk { get; set; }
        [JsonProperty("lun")]
        public string Lun { get; set; }
        [JsonProperty("diskSizeGB")]
        public int DiskSizeGB { get; set; }
    }

    public class OSDisk
    {
        [JsonProperty("caching")]
        public string caching { get; set; }
        [JsonProperty("createOption")]
        public string CreateOption { get; set; }
        [JsonProperty("name")]
        public string Name { get; set; }
        [JsonProperty("image")]
        public VirtualHardDisk SourceImage { get; set; }
        [JsonProperty("vhd")]
        public VirtualHardDisk VirtualHardDisk { get; set; }
        [JsonProperty("osType")]
        public string OperatingSystemType { get; set; }
    }
    public class VirtualHardDisk
    {
        [JsonProperty("uri")]
        public string Uri { get; set; }
    }
    public class WindowsConfiguration
    {
        [JsonProperty("additionalUnattendContent")]
        public AdditionalUnattendContent AdditionalUnattendContent { get; set; }
        [JsonProperty("enableAutomaticUpdates")]
        public bool? EnableAutomaticUpdates { get; set; }
        [JsonProperty("provisionVMAgent")]
        public bool? ProvisionVMAgent { get; set; }
        [JsonProperty("timeZone")]
        public string TimeZone { get; set; }
        [JsonProperty("winRM")]
        public WinRMConfiguration WinRMConfiguration { get; set; }
    }

    public class AdditionalUnattendContent
    {
        [JsonProperty("componentName")]
        public string ComponentName { get; set; }
        [JsonProperty("content")]
        public string Content { get; set; }
        [JsonProperty("passName")]
        public string PassName { get; set; }
        [JsonProperty("settingName")]
        public string SettingName { get; set; }
    }

    public class WinRMConfiguration
    {
        [JsonProperty("listeners")]
        public WinRMListener[] Listeners { get; set; }
    }
    public class WinRMListener
    {
        [JsonProperty("certificateUrl")]
        public string Certificateurl { get; set; }
        [JsonProperty("protocol")]
        public string Protocol { get; set; }
    }
}
