﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.Samples.CloudSolutionProviderManagement.Azure.DataModel.Network
{
    public class LoadBalancerList
    {
        [JsonProperty("value")]
        public LoadBalancer[] LoadBalancers { get; set; }
        [JsonProperty("nextLink")]
        public string NextLink { get; set; }
    }

    public class LoadBalancer
    {
        [JsonProperty("id")]
        public string Id { get; set; }
        [JsonProperty("name")]
        public string Name { get; set; }
        [JsonProperty("type")]
        public string Type { get; set; }
        [JsonProperty("location")]
        public string Location { get; set; }
        [JsonProperty("tags")]
        public IDictionary<string, string> Tags { get; set; }
        [JsonProperty("etag")]
        public string Etag { get; set; }
        [JsonProperty("properties")]
        public LoadBalancerProperties Properties { get; set; }
    }

    public class LoadBalancerProperties
    {
        [JsonProperty("backendAddressPools")]
        public BackendAddressPool[] BackendAddressPools { get; set; }
        [JsonProperty("frontendIPConfiguration")]
        public FrontendIpConfiguration[] FrontendIpconfigurations { get; set; }
        [JsonProperty("inboundNatRules")]
        public InboundNatRule[] InboundNatRules { get; set; }
        [JsonProperty("loadBalancingRule")]
        public LoadBalancingRule[] LoadBalancingRules { get; set; }
        [JsonProperty("probes")]
        public Probe[] Probes { get; set; }
        [JsonProperty("provisioningState")]
        public string ProvisioningState { get; set; }
    }

    public class InboundNatRule
    {
        [JsonProperty("id")]
        public string Id { get; set; }
        [JsonProperty("etag")]
        public string Etag { get; set; }
        [JsonProperty("name")]
        public string Name { get; set; }
        [JsonProperty("properties")]
        public InboundNatRuleProperties Properties { get; set; }
    }

    public class InboundNatRuleProperties
    {
        [JsonProperty("backendIPConfiguration")]
        public ResourceId backendIPPConfiguration { get; set; }
        [JsonProperty("backendPort")]
        public int BackendPort { get; set; }
        [JsonProperty("enableFloatingIP")]
        public bool EnableFloatingIP { get; set; }
        [JsonProperty("frontendIPconfiguration")]
        public ResourceId FrontendIPconfiguration { get; set; }
        [JsonProperty("frontendPort")]
        public int FrontendPort { get; set; }
        [JsonProperty("idleTimeoutInMinutes")]
        public int? IdleTimeoutInMinutes { get; set; }
        [JsonProperty("protocol")]
        public string Protocol { get; set; }
        [JsonProperty("provisioningState")]
        public string ProvisioningState { get; set; }
    }

    public class LoadBalancingRule
    {
        [JsonProperty("id")]
        public string Id { get; set; }
        [JsonProperty("etag")]
        public string Etag { get; set; }
        [JsonProperty("name")]
        public string Name { get; set; }
        [JsonProperty("properties")]
        public LoadBalancingRuleProperties properties { get; set; }
    }

    public class LoadBalancingRuleProperties
    {
        [JsonProperty("backendAddressPool")]
        public ResourceId BackendAddressPool { get; set; }
        [JsonProperty("backendPort")]
        public int backendPort { get; set; }
        [JsonProperty("enableFloatingIP")]
        public bool EnableFloatingIP { get; set; }
        [JsonProperty("frontendIPConfiguration")]
        public ResourceId FrontendIPConfiguration { get; set; }
        [JsonProperty("frontendPort")]
        public int FrontendPort { get; set; }
        [JsonProperty("idleTimeoutInMinutes")]
        public int? IdleTimeoutInMinutes { get; set; }
        [JsonProperty("loadDistribution")]
        public string LoadDistribution { get; set; }
        [JsonProperty("probe")]
        public ResourceId Probe { get; set; }
        [JsonProperty("protocol")]
        public string Protocol { get; set; }
        [JsonProperty("provisioningState")]
        public string ProvisioningState { get; set; }
    }
    public class Probe
    {
        [JsonProperty("id")]
        public string Id { get; set; }
        [JsonProperty("etag")]
        public string Etag { get; set; }
        [JsonProperty("name")]
        public string Name { get; set; }
        [JsonProperty("properties")]
        public ProbeProperties Properties { get; set; }
    }

    public class ProbeProperties
    {
        [JsonProperty("intervalInSeconds")]
        public int IntervalInSeconds { get; set; }
        [JsonProperty("loadBalancingRules")]
        public ResourceId[] LoadBalancingRules { get; set; }
        [JsonProperty("numberOfProbes")]
        public int NumberOfProbes { get; set; }
        [JsonProperty("port")]
        public int Port { get; set; }
        [JsonProperty("protocol")]
        public string Protocol { get; set; }
        [JsonProperty("provisioningState")]
        public string ProvisioningState { get; set; }
        [JsonProperty("requestPath")]
        public string RequestPath { get; set; }
    }

    public class FrontendIpConfiguration
    {
        [JsonProperty("id")]
        public string Id { get; set; }
        [JsonProperty("etag")]
        public string Etag { get; set; }
        [JsonProperty("name")]
        public string Name { get; set; }
        [JsonProperty("properties")]
        public FrontendIPConfigurationProperties Properties { get; set; }
    }

    public class FrontendIPConfigurationProperties
    {
        [JsonProperty("inboundNatRules")]
        public ResourceId[] InboundNatRules { get; set; }
        [JsonProperty("loadBalancingRules")]
        public ResourceId[] LoadBalancingRules { get; set; }
        [JsonProperty("privateIpAddress")]
        public string PrivateIpAddress { get; set; }
        [JsonProperty("privateIpAllocationMethod")]
        public string PrivateIpAllocationMethod { get; set; }
        [JsonProperty("provisioningState")]
        public string ProvisioningState { get; set; }
        [JsonProperty("publicIpAddress")]
        public ResourceId PublicIpAddress { get; set; }
        [JsonProperty("subnet")]
        public ResourceId subnet { get; set; }
    }

    public class BackendAddressPool
    {
        [JsonProperty("id")]
        public string Id { get; set; }
        [JsonProperty("etag")]
        public string Etag { get; set; }
        [JsonProperty("name")]
        public string Name { get; set; }
        [JsonProperty("properties")]
        public BackendAddressPoolProperties Properties { get; set; }
    }

    public class BackendAddressPoolProperties
    {
        [JsonProperty("backendIpConfigurations")]
        public ResourceId[] backendIpConfigurations { get; set; }
        [JsonProperty("loadBalancingRules")]
        public ResourceId[] LoadbalancingRules { get; set; }
        [JsonProperty("provisioningState")]
        public string ProvisioningState { get; set; }
    }
}
