﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.Samples.CloudSolutionProviderManagement.Azure.DataModel.ResourceManager
{
    public class DeploymentList
    {
        [JsonProperty("value")]
        public Deployment[] Deployments { get; set; }
        [JsonProperty("nextLink")]
        public string NextLink { get; set; }
    }

    public class Deployment
    {
        [JsonProperty("id")]
        public string id { get; set; }
        [JsonProperty("name")]
        public string Name { get; set; }
        [JsonProperty("properties")]
        public DeploymentProperties Properties { get; set; }
    }

    public class DeploymentProperties
    {
        [JsonProperty("mode")]
        public string DeploymentMode { get; set; }
        [JsonProperty("parameters")]
        public IDictionary<string, TemplateParameter> Parameters { get; set; }
        [JsonProperty("parametersLink")]
        public ParametersLink ParamtersLink { get; set; }
        [JsonProperty("template")]
        public string Template { get; set; }
        [JsonProperty("templateLink")]
        public TemplateLink TemplateLink { get; set; }
        [JsonProperty("correlationId")]
        public string CorrelationId { get; set; }
        [JsonProperty("dependencies")]
        public Dependency[] Dependencies { get; set; }
        [JsonProperty("outputs")]
        public IDictionary<string, TemplateParameter> Outputs { get; set; }
        [JsonProperty("providers")]
        public Provider[] Providers { get; set; }
        [JsonProperty("provisioningState")]
        public string ProvisioningState { get; set; }
        [JsonProperty("timeStamp")]
        public DateTime TimeStamp { get; set; }
    }

    public class ParametersLink
    {
        [JsonProperty("contentVersion")]
        public string ContentVersion { get; set; }
        [JsonProperty("uri")]
        public string Uri { get; set; }
    }

    public class TemplateLink
    {
        [JsonProperty("contentVersion")]
        public string ContentVersion { get; set; }
        [JsonProperty("uri")]
        public string Uri { get; set; }
    }

    public class TemplateParameter
    {
        [JsonProperty("type")]
        public string Type { get; set; }

        [JsonProperty("value")]
        public object Value { get; set; }
    }

    public class Dependency
    {
        [JsonProperty("id")]
        public string Id { get; set; }
        [JsonProperty("resourceName")]
        public string ResourceName { get; set; }
        [JsonProperty("resourceType")]
        public string ResourceType { get; set; }
        [JsonProperty("dependsOn")]
        public BasicDependency[] DependsOn { get; set; }
    }

    public class BasicDependency
    {
        [JsonProperty("id")]
        public string Id { get; set; }
        [JsonProperty("resourceName")]
        public string ResourceName { get; set; }
        [JsonProperty("resourceType")]
        public string ResourceType { get; set; }
    }

    public class DeploymentOperationList
    {
        [JsonProperty("value")]
        public DeploymentOperation[] DeploymentOperations { get; set; }
        [JsonProperty("nextLink")]
        public string NextLink { get; set; }
    }

    public class DeploymentOperation
    {
        [JsonProperty("id")]
        public string Id { get; set; }
        [JsonProperty("operationId")]
        public string OperationId { get; set; }
        [JsonProperty("properties")]
        public DeploymentOperationProperties Properties { get; set; }
    }

    public class DeploymentOperationProperties
    {
        [JsonProperty("provisioningState")]
        public string ProvisioningState { get; set; }
        [JsonProperty("statusCode")]
        public string StatusCode { get; set; }
        [JsonProperty("statusMessage")]
        public string StatusMessage { get; set; }
        [JsonProperty("targetResource")]
        public TargetResource TargetResource { get; set; }
        [JsonProperty("timestamp")]
        public DateTime Timestamp { get; set; }
    }

    public class TargetResource
    {
        [JsonProperty("id")]
        public string Id { get; set; }
        [JsonProperty("resourceName")]
        public string ResourceName { get; set; }
        [JsonProperty("resourceType")]
        public string ResourceType { get; set; }
    }

    public class CreateUpdateDeploymentParameters
    {
        [JsonProperty("properties")]
        public CreateUpdateDeploymentParametersProperties Properties { get; set; }
    }

    public class CreateUpdateDeploymentParametersProperties
    {
        [JsonProperty("mode")]
        public string DeploymentMode { get; set; }
        [JsonProperty("parameters")]
        public IDictionary<string, DeploymentParameter> Parameters { get; set; }
        [JsonProperty("parametersLink")]
        public ParametersLink ParamtersLink { get; set; }
        [JsonProperty("template")]
        public string Template { get; set; }
        [JsonProperty("templateLink")]
        public TemplateLink TemplateLink { get; set; }
    }

    public class DeploymentParameter
    {
        [JsonProperty("value")]
        public string Value { get; set; }
    }
}
