﻿using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderManagement.AzureActiveDirectory.DataModel;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.Samples.CloudSolutionProviderManagement.AzureActiveDirectory
{
    /// <summary>
    /// 
    /// Reference: https://msdn.microsoft.com/Library/Azure/Ad/Graph/api/api-catalog
    /// </summary>
    public static class AzureActiveDirectoryManager
    {
        #region Contract
        /// <summary>
        /// 
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <returns></returns>
        public static async Task<Contract> GetCustomers(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler)
        {
            Contract customerContracts=null;
            try
            {
                var path=string.Format("{0}/myorganization/contracts?api-version={1}", Constants.AzureADGraphAPIUrl, Constants.AzureADGraphAPIVersion);

                Uri uri = EnsureAbsoluteUri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await GraphApiInvoke(uri, cacheInfo, "get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        customerContracts = JsonConvert.DeserializeObject<Contract>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch(Exception ex)
            {
                throw ex;
            }

            return customerContracts;
        }
        #endregion

        #region User
        public static async Task<Users> GetUsers(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string tenantId)
        {
            Users users = null;
            try
            {
                var path = string.Format("{0}/{1}/users?api-version={2}", Constants.AzureADGraphAPIUrl, tenantId, Constants.AzureADGraphAPIVersion);

                Uri uri = EnsureAbsoluteUri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await GraphApiInvoke(uri, cacheInfo, "get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        users = JsonConvert.DeserializeObject<Users>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return users;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <param name="tenantId"></param>
        /// <param name="newUser"></param>
        /// <returns></returns>
        public static async Task<User> NewUser(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string tenantId, CreateUserParameters newUser)
        {
            User user = null;
            try
            {
                var path = string.Format("{0}/{1}/users?api-version={2}", Constants.AzureADGraphAPIUrl, tenantId, Constants.AzureADGraphAPIVersion);

                Uri uri = EnsureAbsoluteUri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(newUser), Encoding.UTF8, Constants.JsonContentType); 
                using (HttpResponseMessage response = await GraphApiInvoke(uri, cacheInfo, "Post", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        user = JsonConvert.DeserializeObject<User>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return user;
        }
        #endregion

        #region Role
        public static async Task<DirectoryRoles> GetDirectoryRoles(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string tenantId)
        {
            DirectoryRoles roles = null;
            try
            {
                var path = string.Format("{0}/{1}/directoryRoles?api-version={2}", Constants.AzureADGraphAPIUrl, tenantId, Constants.AzureADGraphAPIVersion);

                Uri uri = EnsureAbsoluteUri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await GraphApiInvoke(uri, cacheInfo, "get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        roles = JsonConvert.DeserializeObject<DirectoryRoles>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return roles;
        }

        //public static async void GetDirectoryRoleMembers(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string tenantId, string roleObjectId)
        //{
        //    DirectoryRoles roles = null;
        //    try
        //    {
        //        var path = string.Format("{0}/{1}/directoryRoles/{2}/members?api-version={3}", Constants.AzureADGraphAPIUrl, tenantId, roleObjectId, Constants.AzureADGraphAPIVersion);

        //        Uri uri = EnsureAbsoluteUri(path);
        //        HttpContent payload = null;
        //        using (HttpResponseMessage response = await GraphApiInvoke(uri, cacheInfo, "get", handler, payload))
        //        {
        //            if (response.IsSuccessStatusCode)
        //            {
        //                string jsonResult = await response.Content.ReadAsStringAsync();
        //                //roles = JsonConvert.DeserializeObject<DirectoryRoles>(jsonResult);
        //            }
        //            else
        //            {
        //                throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
        //            }
        //        }
        //    }
        //    catch (Exception ex)
        //    {
        //        throw ex;
        //    }

        //    //return roles;
        //}

        public static async Task<bool> AddDirectoryRoleMember(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string tenantId, string roleObjectId, AddRoleMemberParameters parameter)
        {
            bool success = false; 
            try
            {
                var path = string.Format("{0}/{1}/directoryRoles/{2}/$links/members?api-version={3}", Constants.AzureADGraphAPIUrl, tenantId, roleObjectId, Constants.AzureADGraphAPIVersion);

                Uri uri = EnsureAbsoluteUri(path);
                HttpContent payload = new StringContent(JsonConvert.SerializeObject(parameter), Encoding.UTF8, Constants.JsonContentType); 
                using (HttpResponseMessage response = await GraphApiInvoke(uri, cacheInfo, "Post", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        success = true;
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return success;
        }
        #endregion

        #region Group
        public static async Task<Groups> GetGroups(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string tenantId)
        {
            Groups groups = null;
            try
            {
                var path = string.Format("{0}/{1}/groups?api-version={2}", Constants.AzureADGraphAPIUrl, tenantId, Constants.AzureADGraphAPIVersion);

                Uri uri = EnsureAbsoluteUri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await GraphApiInvoke(uri, cacheInfo, "get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        groups = JsonConvert.DeserializeObject<Groups>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return groups;
        }
        #endregion

        #region Application
        public static async Task<Applications> GetApplications(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string tenantId)
        {
            Applications apps = null;
            try
            {
                var path = string.Format("{0}/{1}/applications?api-version={2}", Constants.AzureADGraphAPIUrl, tenantId, Constants.AzureADGraphAPIVersion);

                Uri uri = EnsureAbsoluteUri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await GraphApiInvoke(uri, cacheInfo, "get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        apps = JsonConvert.DeserializeObject<Applications>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return apps;
        }
        #endregion

        #region Service Principal
        public static async Task<ServicePrincipals> GetServicePrincipals(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string tenantId)
        {
            ServicePrincipals sps = null;
            try
            {
                var path = string.Format("{0}/{1}/servicePrincipals?api-version={2}", Constants.AzureADGraphAPIUrl, tenantId, Constants.AzureADGraphAPIVersion);

                Uri uri = EnsureAbsoluteUri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await GraphApiInvoke(uri, cacheInfo, "get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        sps = JsonConvert.DeserializeObject<ServicePrincipals>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return sps;
        }
        #endregion

        #region Domain Management
        /// <summary>
        /// http://blogs.msdn.com/b/aadgraphteam/archive/2015/07/24/announcing-the-preview-of-domain-management-api.aspx
        /// </summary>
        /// <param name="cacheInfo"></param>
        /// <param name="handler"></param>
        /// <param name="tenantId"></param>
        /// <returns></returns>
        public static async Task<Domains> GetDomains(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string tenantId)
        {
            Domains domains = null;
            try
            {
                var path = string.Format("{0}/{1}/domains?api-version={2}", Constants.AzureADGraphAPIUrl, tenantId, Constants.AzureADGraphAPIPreviewVersion);

                Uri uri = EnsureAbsoluteUri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await GraphApiInvoke(uri, cacheInfo, "get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        domains = JsonConvert.DeserializeObject<Domains>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return domains;
        }
        #endregion

        #region TenantDetail
        public static async Task<TenantDetails> GetTenantDetails(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string tenantId)
        {
            TenantDetails tenantDetails = null;
            try
            {
                var path = string.Format("{0}/{1}/tenantDetails?api-version={2}", Constants.AzureADGraphAPIUrl, tenantId, Constants.AzureADGraphAPIVersion);

                Uri uri = EnsureAbsoluteUri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await GraphApiInvoke(uri, cacheInfo, "get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        tenantDetails = JsonConvert.DeserializeObject<TenantDetails>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return tenantDetails;
        }

        public static async Task<TenantDetail> GetTenantDetailById(AuthTokenCacheInfo cacheInfo, DelegatingHandler handler, string tenantId, string objectId)
        {
            TenantDetail tenantDetail = null;
            try
            {
                var path = string.Format("{0}/{1}/tenantDetails/{3}?api-version={2}", Constants.AzureADGraphAPIUrl, tenantId, Constants.AzureADGraphAPIVersion, objectId);

                Uri uri = EnsureAbsoluteUri(path);
                HttpContent payload = null;
                using (HttpResponseMessage response = await GraphApiInvoke(uri, cacheInfo, "get", handler, payload))
                {
                    if (response.IsSuccessStatusCode)
                    {
                        string jsonResult = await response.Content.ReadAsStringAsync();
                        tenantDetail = JsonConvert.DeserializeObject<TenantDetail>(jsonResult);
                    }
                    else
                    {
                        throw new InvalidOperationException(await response.Content.ReadAsStringAsync());
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return tenantDetail;
        }
        #endregion


        public static Uri EnsureAbsoluteUri(string path)
        {
            Uri ret;
            if (Uri.TryCreate(path, UriKind.Absolute, out ret))
            {
                return ret;
            }

            return new Uri(new Uri(Constants.AzureADGraphAPIUrl), path);
        }

        public static async Task<HttpResponseMessage> GraphApiInvoke(Uri uri, AuthTokenCacheInfo cacheInfo, string verb, DelegatingHandler handler, HttpContent content)
        {
            using (var client = new HttpClient(handler))
            {
                HttpResponseMessage response = null;
                client.DefaultRequestHeaders.Add("Authorization", cacheInfo.GetAuthorizationHeader());
                client.DefaultRequestHeaders.Add("Accept", Constants.JsonContentType);
                if (String.Equals(verb, "get", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.GetAsync(uri);
                }
                else if (String.Equals(verb, "delete", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.DeleteAsync(uri);
                }
                else if (String.Equals(verb, "post", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.PostAsync(uri, content);
                }
                else if (String.Equals(verb, "put", StringComparison.OrdinalIgnoreCase))
                {
                    response = await client.PutAsync(uri, content);
                }
                else if (String.Equals(verb, "patch", StringComparison.OrdinalIgnoreCase))
                {
                    using (var message = new HttpRequestMessage(new HttpMethod("PATCH"), uri))
                    {
                        message.Content = content;
                        response = await client.SendAsync(message).ConfigureAwait(false);
                    }
                }
                else
                {
                    throw new InvalidOperationException(String.Format("Invalid http verb {0}!", verb));
                }
                return response;
            }
        }
    }
}
