﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter.DataModel
{
    /// <summary>
    /// Class represents customer information returned by partner center APIs
    /// </summary>
    public class Customer
    {
        [JsonProperty("id")]
        public string Id { get; set; }

        [JsonProperty("identity")]
        public CustomerIdentity CustomerIdentity { get; set; }

        [JsonProperty("is_test")]
        public bool IsTest { get; set; }

        [JsonProperty("links")]
        public CustomerLinks CustomerLinks { get; set; }

        [JsonProperty("object_type")]
        public string ObjectType { get; set; }
    }

    /// <summary>
    /// Class represents customer identity information returned by partner center APIs
    /// </summary>
    public class CustomerIdentity
    {
        [JsonProperty("provider")]
        public string Provider { get; set; }

        [JsonProperty("type")]
        public string Type { get; set; }

        [JsonProperty("data")]
        public CustomerData CustomerData { get; set; }
    }

    /// <summary>
    /// Class represents customer related data returned by partner center APIs
    /// </summary>
    public class CustomerData
    {
        [JsonProperty("tid")]
        public string Tid { get; set; }

        [JsonProperty("etid")]
        public string Etid { get; set; }

        [JsonProperty("eoid")]
        public string Eoid { get; set; }
    }

    /// <summary>
    /// Class represents Uri for getting customer profile information
    /// </summary>
    public class ProfileLink
    {

        [JsonProperty("href")]
        public string Href { get; set; }

        [JsonProperty("method")]
        public string Method { get; set; }
    }

    /// <summary>
    /// Class represents Uri for getting customer address information
    /// </summary>
    public class AddressLink
    {

        [JsonProperty("href")]
        public string Href { get; set; }

        [JsonProperty("method")]
        public string Method { get; set; }
    }

    /// <summary>
    /// Class represents Uris for getting customer related information returned by partner center APIs
    /// </summary>
    public class CustomerLinks
    {

        [JsonProperty("self")]
        public SelfLink Self { get; set; }

        [JsonProperty("profiles")]
        public ProfileLink ProfileLink { get; set; }

        [JsonProperty("addresses")]
        public AddressLink AddressLink { get; set; }
    }
}
