﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter.DataModel
{
    public class OfferCatalog
    {
        public IEnumerable<OfferGroup> OfferGroupCollection { get; set; }

        private static OfferCatalog catalog { get; set; }

        private static object singletonObject=new object();

        private OfferCatalog()
        {
            OfferGroupCollection = new List<OfferGroup>
            {
                new OfferGroup
                {
                    OfferCategory=OfferCategory.Azure,
                    Offers=new List<Offer>
                    {
                        new Offer
                        {
                            Name="Azure Cloud Solution Provider",
                            Uri = "/fbf178a5-144e-46d1-aa81-612c2d3f97f4/offers/MS-AZR-0145P",
                            Id = "MS-AZR-0145P",
                            Minimum = 1,
                            Maximum = 999999999,
                        }
                    }
                },
                new OfferGroup
                {
                    OfferCategory=OfferCategory.IntuneAndOffice,
                    Offers=new List<Offer>
                    {
                        new Offer
                        {
                            Name = "Azure Active Directory Basic",
                            Uri = "/3c95518e-8c37-41e3-9627-0ca339200f53/offers/84A03D81-6B37-4D66-8D4A-FAEA24541538",
                            Id = "84a03d81-6b37-4d66-8d4a-faea24541538",
                            Minimum = 1,
                            Maximum= 10000000,
                        },
                        new Offer
                        {
                            Name = "Azure Active Directory Premium",
                            Uri = "/3c95518e-8c37-41e3-9627-0ca339200f53/offers/16C9F982-A827-4003-A88E-E75DF1927F27",
                            Id = "16c9f982-a827-4003-a88e-e75df1927f27",
                            Minimum = 1,
                            Maximum = 10000000,
                        },
                        new Offer
                        {
                            Name = "Intune",
                            Uri = "/3c95518e-8c37-41e3-9627-0ca339200f53/offers/51E95709-DC35-4780-9040-22278CB7C0E1",
                            Id = "51e95709-dc35-4780-9040-22278cb7c0e1",
                            Minimum = 1,
                            Maximum = 10000000,
                        },
                        new Offer
                        {
                            Name = "Enterprise Mobility Suite",
                            Uri = "/3c95518e-8c37-41e3-9627-0ca339200f53/offers/79C29AF7-3CD0-4A6F-B182-A81E31DEC84E",
                            Id = "79c29af7-3cd0-4a6f-b182-a81e31dec84e",
                            Minimum = 1,
                            Maximum= 10000000,
                        },
                        new Offer
                        {
                            Name = "Exchange Online (Plan 1)",
                            Uri = "/3c95518e-8c37-41e3-9627-0ca339200f53/offers/195416C1-3447-423A-B37B-EE59A99A19C4",
                            Id = "195416c1-3447-423a-b37b-ee59a99a19c4",
                            Minimum = 1,
                            Maximum = 10000000,
                            ConvertibleOffers = new List<string>
                            {
                                "796B6B5F-613C-4E24-A17C-EBA730D49C02",
                                "031C9E47-4802-4248-838E-778FB1D2CC05",
                                "BD938F12-058F-4927-BBA3-AE36B1D2501C",
                                "91FD106F-4B2C-4938-95AC-F54F74E9A239"
                            }
                        },
                        new Offer
                        {
                            Name = "Lync Online (Plan 1)",
                            Uri = "/3c95518e-8c37-41e3-9627-0ca339200f53/offers/2F707C7C-2433-49A5-A437-9CA7CF40D3EB",
                            Id = "aca0c06c-890d-4abb-83cf-bc519a2565e5",
                            Minimum = 1,
                            Maximum = 10000000,
                            ConvertibleOffers = new List<string>
                            {
                                "796B6B5F-613C-4E24-A17C-EBA730D49C02",
                                "031C9E47-4802-4248-838E-778FB1D2CC05",
                                "BD938F12-058F-4927-BBA3-AE36B1D2501C",
                                "91FD106F-4B2C-4938-95AC-F54F74E9A239"
                            }
                        },
                        new Offer
                        {
                            Name = "Office 365 Enterprise E1",
                            Uri = "/3c95518e-8c37-41e3-9627-0ca339200f53/offers/91FD106F-4B2C-4938-95AC-F54F74E9A239",
                            Id = "91fd106f-4b2c-4938-95ac-f54f74e9a239",
                            Minimum = 1,
                            Maximum = 10000000
                        },
                        new Offer
                        {
                            Name = "Exchange Online Archiving for Exchange Online",
                            Uri = "/3c95518e-8c37-41e3-9627-0ca339200f53/offers/2828BE95-46BA-4F91-B2FD-0BEF192ECF60",
                            Id = "2828be95-46ba-4f91-b2fd-0bef192ecf60",
                            Minimum = 1,
                            Maximum = 10000000,
                            DependentOffers = new List<string>
                            {
                                "35A36B80-270A-44BF-9290-00545D350866",
                                "6FBAD345-B7DE-42A6-B6AB-79B363D0B371",
                                "91FD106F-4B2C-4938-95AC-F54F74E9A239",
                                "837F8912-2321-41D1-A50A-9F3C4F43C93B",
                                "195416C1-3447-423A-B37B-EE59A99A19C4"
                            }
                        },
                    }
                }
            };
        }

        public static OfferCatalog Catalog
        {
            get
            {
                if(catalog!= null)
                {
                    return catalog;
                }

                lock (singletonObject)
                {
                    if(catalog== null)
                    {
                        catalog = new OfferCatalog();
                    }
                }

                return catalog;
            }
        }
    }
}
