﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter.DataModel
{
    /// <summary>
    /// Class represents order line item
    /// </summary>
    public class LineItem
    {

        [JsonProperty("line_item_number")]
        public int LineItemNumber { get; set; }

        [JsonProperty("offer_uri")]
        public string OfferUri { get; set; }

        [JsonProperty("quantity")]
        public int Quantity { get; set; }

        [JsonProperty("resulting_subscription_uri")]
        public string ResultingSubscriptionUri { get; set; }
    }

    /// <summary>
    /// Class represents Uri for patching the order line item
    /// </summary>
    public class PatchLink
    {

        [JsonProperty("href")]
        public string Href { get; set; }

        [JsonProperty("method")]
        public string Method { get; set; }
    }

    /// <summary>
    /// Class represents Uri information for getting payer information
    /// </summary>
    public class PayerLink
    {

        [JsonProperty("href")]
        public string Href { get; set; }

        [JsonProperty("method")]
        public string Method { get; set; }
    }

    /// <summary>
    /// Class represents Uri for getting receipent information
    /// </summary>
    public class RecipientLink
    {

        [JsonProperty("href")]
        public string Href { get; set; }

        [JsonProperty("method")]
        public string Method { get; set; }
    }

    public class OrderItemLinks
    {

        [JsonProperty("self")]
        public SelfLink Self { get; set; }

        [JsonProperty("patch")]
        public PatchLink PatchLink { get; set; }

        [JsonProperty("payer")]
        public PayerLink PayerLink { get; set; }

        [JsonProperty("recipient")]
        public RecipientLink RecipientLink { get; set; }
    }

    /// <summary>
    /// Class represents order item
    /// </summary>
    public class OrderItem
    {

        [JsonProperty("id")]
        public string Id { get; set; }

        [JsonProperty("recipient_customer_id")]
        public string RecipientCustomerId { get; set; }

        [JsonProperty("etag")]
        public string Etag { get; set; }

        [JsonProperty("creation_date")]
        public string CreationDate { get; set; }

        [JsonProperty("line_items")]
        public LineItem[] LineItems { get; set; }

        [JsonProperty("object_type")]
        public string ObjectType { get; set; }

        [JsonProperty("contract_version")]
        public string ContractVersion { get; set; }

        [JsonProperty("links")]
        public OrderItemLinks OrderItemLinks { get; set; }
    }

    /// <summary>
    /// Class represents Uri for adding order item
    /// </summary>
    public class AddLink
    {

        [JsonProperty("href")]
        public string Href { get; set; }

        [JsonProperty("method")]
        public string Method { get; set; }
    }

    /// <summary>
    /// Class represents Uri for managing order
    /// </summary>
    public class OrderLinks
    {

        [JsonProperty("self")]
        public SelfLink Self { get; set; }

        [JsonProperty("add")]
        public AddLink AddLink { get; set; }
    }


    /// <summary>
    /// Class represents order information returned by partner center RESTfule APIs
    /// </summary>
    public class Orders
    {

        [JsonProperty("object_type")]
        public string ObjectType { get; set; }

        [JsonProperty("contract_version")]
        public string ContractVersion { get; set; }

        [JsonProperty("items")]
        public OrderItem[] Items { get; set; }

        [JsonProperty("total_count")]
        public int TotalCount { get; set; }

        [JsonProperty("links")]
        public OrderLinks OrderLinks { get; set; }
    }
}
