﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter.DataModel
{
    /// <summary>
    /// Class encapulates the Authorization Token for invoking Partner Center APIs
    /// </summary>
    public class PartnerCenterAuthToken
    {
        /// <summary>
        /// Captures when the token expires
        /// </summary>
        private DateTimeOffset ExpiresOn { get; set; }

        /// <summary>
        /// Access token
        /// </summary>
        public string AccessToken { get; private set; }

        /// <summary>
        /// Constructor for getting an authorization token
        /// </summary>
        /// <param name="access_Token">access token</param>
        /// <param name="expires_in">number of seconds the token is valid for</param>
        public PartnerCenterAuthToken(string access_Token, DateTimeOffset expiresOn)
        {
            this.AccessToken = access_Token;
            this.ExpiresOn = expiresOn;
        }

        /// <summary>
        /// Returns true if the authorization token is near expiry
        /// </summary>
        /// <returns>true if the authorization token is near expiry</returns>
        public bool IsExpired()
        {
            //// if token is expiring in the next minute or expired, return true
            return ExpiresOn <= DateTimeOffset.UtcNow;
        }
    }
}
