﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure.DataModel.SQLDatabase;
using System;
using System.ComponentModel.Composition;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.Azure.AzureSQL
{
    [Export(typeof(DatabaseViewModel))]
    public class DatabaseViewModel : Screen
    {
        private readonly IWindowManager windowManager;
        private readonly IEventAggregator eventAggregator;
        private string tenantId;
        private string subscriptionId;
        private string resourceGroupName;
        private string serverName;
        private Database dbItem;
        private LoginType authType;

        [ImportingConstructor]
        public DatabaseViewModel(IWindowManager winMgr, IEventAggregator evt, string tId, string sId, string rgName, string sName, Database item, LoginType loginType)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            tenantId = tId;
            subscriptionId = sId;
            resourceGroupName = rgName;
            serverName = sName;
            dbItem = item;
            authType = loginType;
        }

        public string DatabaseId
        {
            get
            {
                return dbItem.Id;
            }
        }

        public string DatabaseName
        {
            get
            {
                return dbItem.Name;
            }
        }

        public string Location
        {
            get
            {
                return dbItem.Location;
            }
        }

        public string DatabaseType
        {
            get
            {
                return dbItem.Type;
            }
        }

        public string Edition
        {
            get
            {
                return dbItem.Properties.Edition;
            }
        }

        public string ServiceObjective
        {
            get
            {
                return dbItem.Properties.ServiceObjective;
            }
        }

        public string Collation
        {
            get
            {
                return dbItem.Properties.Collation;
            }
        }

        public long MaxSizeBytes
        {
            get
            {
                return dbItem.Properties.MaxSizeBytes;
            }
        }

        public DateTime CreationDate
        {
            get
            {
                return dbItem.Properties.CreationDate;
            }
        }

        public DateTime? EarliestRestoreDate
        {
            get
            {
                return dbItem.Properties.EarliestRestoreDate;
            }
        }

        public string DefaultSecondaryLocation
        {
            get
            {
                return dbItem.Properties.DefaultSecondaryLocation;
            }
        }
           
        public void Delete()
        {

        }

        public void ViewUsageMetric()
        {
            windowManager.ShowDialog(new UsageMetricListViewModel(windowManager, eventAggregator, tenantId, subscriptionId, resourceGroupName, serverName, dbItem.Name, authType));
        }
    }
}
