﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure.DataModel.SQLDatabase;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.Azure.AzureSQL
{
    [Export(typeof(ElasticPoolListViewModel))]
    public class ElasticPoolListViewModel : Screen
    {
        private readonly IWindowManager windowManager;
        private readonly IEventAggregator eventAggregator;
        private string tenantId;
        private string subscriptionId;
        private LoginType authType;
        private string resourceGroupName;
        private string serverName;
        private ElasticPoolList poolList = null;
        private readonly BackgroundWorker worker = new BackgroundWorker();
        private Visibility showLoadIndicator = Visibility.Collapsed;
        private ObservableCollection<ElasticPoolViewModel> pools = null;

        [ImportingConstructor]
        public ElasticPoolListViewModel(IWindowManager winMgr, IEventAggregator evt, string tId, string sId, string rgName, string sName, LoginType loginType)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            tenantId = tId;
            subscriptionId = sId;
            resourceGroupName = rgName;
            serverName = sName;
            authType = loginType;
            worker.DoWork += Worker_DoWork;
            worker.RunWorkerCompleted += Worker_RunWorkerCompleted;
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }

        void Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ShowLoadIndicator = Visibility.Collapsed;
            LoadElasticPools();
        }

        void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            AuthTokenCacheInfo cacheInfo = null;
            if (authType == LoginType.Interactive)
            {
                Task<AuthTokenCacheInfo> cacheInfoTask = App.CurrentAppContext.AuthManager.GetToken(Constants.ClientId, tenantId, "https://management.azure.com/");
                cacheInfoTask.Wait();
                cacheInfo = cacheInfoTask.Result;
            }
            else if (authType == LoginType.Spn)
            {
                cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(tenantId, App.CurrentAppContext.Settings.ManagementAppId, App.CurrentAppContext.Settings.ManagementAppSecret, "https://management.azure.com/");
            }
            if (cacheInfo != null)
            {
                Task<ElasticPoolList> poolListTask = AzureResourceManager.ListAzureSqlElasticPoolsByServer(cacheInfo, handler, subscriptionId, resourceGroupName, serverName);
                poolListTask.Wait();
                poolList = poolListTask.Result;
            }
        }

        public Visibility ShowLoadIndicator
        {
            get
            {
                return showLoadIndicator;
            }
            set
            {
                showLoadIndicator = value;
                NotifyOfPropertyChange(() => ShowLoadIndicator);
            }
        }

        public ObservableCollection<ElasticPoolViewModel> Pools
        {
            get
            {
                return pools;
            }
            set
            {
                pools = value;
                NotifyOfPropertyChange(() => Pools);
            }
        }

        public void LoadElasticPools()
        {
            Pools = new ObservableCollection<ElasticPoolViewModel>();

            if (poolList != null)
            {
                foreach (ElasticPool p in poolList.ElasticPools)
                {
                    pools.Add(new ElasticPoolViewModel(windowManager, eventAggregator, tenantId, subscriptionId, resourceGroupName, serverName, p, authType));
                }
            }
        }

        public void Reload()
        {
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }
    }
}
