﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure.DataModel.Network;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.Azure.Network
{
    [Export(typeof(VirtualNetworkListViewModel))]
    public class VirtualNetworkListViewModel : Screen
    {
        private readonly IWindowManager windowManager;
        private readonly IEventAggregator eventAggregator;
        private string tenantId;
        private string subscriptionId;
        private LoginType authType;
        private string resourceGroupName;
        private VirtualNetworkList vnList = null;
        private readonly BackgroundWorker worker = new BackgroundWorker();
        private Visibility showLoadIndicator = Visibility.Collapsed;
        private ObservableCollection<VirtualNetworkViewModel> virtualNetworks = null;

        [ImportingConstructor]
        public VirtualNetworkListViewModel(IWindowManager winMgr, IEventAggregator evt, string tId, string sId, string rgName, LoginType loginType)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            tenantId = tId;
            subscriptionId = sId;
            resourceGroupName = rgName;
            authType = loginType;
            worker.DoWork += Worker_DoWork;
            worker.RunWorkerCompleted += Worker_RunWorkerCompleted;
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }

        void Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ShowLoadIndicator = Visibility.Collapsed;
            LoadVirtualNetworks();
        }

        void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            AuthTokenCacheInfo cacheInfo = null;
            if (authType == LoginType.Interactive)
            {
                Task<AuthTokenCacheInfo> cacheInfoTask = App.CurrentAppContext.AuthManager.GetToken(Constants.ClientId, tenantId, "https://management.azure.com/");
                cacheInfoTask.Wait();
                cacheInfo = cacheInfoTask.Result;
            }
            else if (authType == LoginType.Spn)
            {
                cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(tenantId, App.CurrentAppContext.Settings.ManagementAppId, App.CurrentAppContext.Settings.ManagementAppSecret, "https://management.azure.com/");
            }
            if (cacheInfo != null)
            {
                Task<VirtualNetworkList> vnListTask = AzureResourceManager.GetVirtualNetworkListByResourceGroup(cacheInfo, handler, subscriptionId, resourceGroupName);
                vnListTask.Wait();
                vnList = vnListTask.Result;
            }
        }

        public Visibility ShowLoadIndicator
        {
            get
            {
                return showLoadIndicator;
            }
            set
            {
                showLoadIndicator = value;
                NotifyOfPropertyChange(() => ShowLoadIndicator);
            }
        }

        public ObservableCollection<VirtualNetworkViewModel> VirtualNetworks
        {
            get
            {
                return virtualNetworks;
            }
            set
            {
                virtualNetworks = value;
                NotifyOfPropertyChange(() => VirtualNetworks);
            }
        }

        public void LoadVirtualNetworks()
        {
            VirtualNetworks = new ObservableCollection<VirtualNetworkViewModel>();

            if (vnList != null)
            {
                foreach (VirtualNetwork v in vnList.VirtualNetworks)
                {
                    virtualNetworks.Add(new VirtualNetworkViewModel(windowManager, eventAggregator, tenantId, subscriptionId, resourceGroupName, v, authType));
                }
            }
        }

        public void Reload()
        {
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }
    }
}
