﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure.DataModel.Network;
using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.Azure.Network
{
    [Export(typeof(VirtualNetworkViewModel))]
    public class VirtualNetworkViewModel : Screen
    {
        private readonly IWindowManager windowManager;
        private readonly IEventAggregator eventAggregator;
        private string tenantId;
        private string subscriptionId;
        private string resourceGroupName;
        private VirtualNetwork vnItem;
        private LoginType authType;

        [ImportingConstructor]
        public VirtualNetworkViewModel(IWindowManager winMgr, IEventAggregator evt, string tId, string sId, string rgName, VirtualNetwork item, LoginType loginType)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            tenantId = tId;
            subscriptionId = sId;
            resourceGroupName = rgName;
            vnItem = item;
            authType = loginType;
        }

        public string VirtualNetworkName
        {
            get
            {
                return vnItem.Name;
            }
        }

        public string AddressSpace
        {
            get {
                if (vnItem.Properties.AddressSpace != null && vnItem.Properties.AddressSpace.AddressPrefixes != null)
                    return string.Concat(vnItem.Properties.AddressSpace.AddressPrefixes);
                else
                    return null;
            }
        }

        public string DNSOptions
        {
            get
            {
                if (vnItem.Properties.DnsOptions != null && vnItem.Properties.DnsOptions.DnsServers != null)
                    return string.Concat(vnItem.Properties.DnsOptions.DnsServers);
                else
                    return null;
            }
        }

        public Subnet[] Subnets
        {
            get
            {
                return vnItem.Properties.Subnets;
            }
        }
    }
}
