﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure.DataModel.ResourceManager;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.Azure.ResourceManagement
{
    [Export(typeof(ResourceGroupListViewModel))]
    public class ResourceGroupListViewModel : Screen
    {
        private readonly IWindowManager windowManager;
        private readonly IEventAggregator eventAggregator;
        private string tenantId;
        private string subscriptionGuid;
        private LoginType authType;
        private ResourceGroupList rgList = null;
        private readonly BackgroundWorker worker = new BackgroundWorker();
        private Visibility showLoadIndicator = Visibility.Collapsed;
        private ObservableCollection<ResourceGroupViewModel> resourceGroups = null;

        [ImportingConstructor]
        public ResourceGroupListViewModel(IWindowManager winMgr, IEventAggregator evt, string tId, string sId, LoginType loginType)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            tenantId = tId;
            subscriptionGuid = sId;
            authType = loginType;
            worker.DoWork += Worker_DoWork;
            worker.RunWorkerCompleted += Worker_RunWorkerCompleted;
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }

        void Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ShowLoadIndicator = Visibility.Collapsed;
            LoadResourceGroups();
        }

        void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            AuthTokenCacheInfo cacheInfo = null;
            if (authType == LoginType.Interactive)
            {
                Task<AuthTokenCacheInfo> cacheInfoTask = App.CurrentAppContext.AuthManager.GetToken(Constants.ClientId, tenantId, "https://management.azure.com/");
                cacheInfoTask.Wait();
                cacheInfo = cacheInfoTask.Result;
            }
            else if (authType == LoginType.Spn)
            {
                cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(tenantId, App.CurrentAppContext.Settings.ManagementAppId, App.CurrentAppContext.Settings.ManagementAppSecret, "https://management.azure.com/");
            }
            if (cacheInfo != null)
            {
                Task<CloudSolutionProviderManagement.Azure.DataModel.ResourceManager.ResourceGroupList> rgListTask = AzureResourceManager.GetResourceGroupList(cacheInfo, handler, subscriptionGuid, null);
                rgListTask.Wait();
                rgList = rgListTask.Result;
            }
        }

        public Visibility ShowLoadIndicator
        {
            get
            {
                return showLoadIndicator;
            }
            set
            {
                showLoadIndicator = value;
                NotifyOfPropertyChange(() => ShowLoadIndicator);
            }
        }

        public ObservableCollection<ResourceGroupViewModel> ResourceGroups
        {
            get
            {
                return resourceGroups;
            }
            set
            {
                resourceGroups = value;
                NotifyOfPropertyChange(() => ResourceGroups);
            }
        }

        public void LoadResourceGroups()
        {
            ResourceGroups = new ObservableCollection<ResourceGroupViewModel>();

            if (rgList != null)
            {
                foreach (ResourceGroup rg in rgList.ResourceGroups)
                {
                    resourceGroups.Add(new ResourceGroupViewModel(windowManager, eventAggregator, tenantId, subscriptionGuid, rg, authType));
                }
            }
        }

        public void Reload()
        {
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }
    }
}
