﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.Azure.AzureSQL;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.Azure.Network;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.Azure.Storage;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.Azure.VirtualMachine;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure.DataModel.ResourceManager;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.Azure.ResourceManagement
{
    [Export(typeof(ResourceGroupViewModel))]
    public class ResourceGroupViewModel : Conductor<object>
    {
        private readonly IWindowManager windowManager;
        private readonly IEventAggregator eventAggregator;
        private string tenantId;
        private string subscriptionId;
        private ResourceGroup rgItem;
        private LoginType authType;
        private readonly BackgroundWorker worker = new BackgroundWorker();

        [ImportingConstructor]
        public ResourceGroupViewModel(IWindowManager winMgr, IEventAggregator evt, string tId, string sId, ResourceGroup item, LoginType loginType)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            tenantId = tId;
            subscriptionId = sId;
            rgItem = item;
            authType = loginType;
        }

        public string ResourceGroupName
        {
            get
            {
                return rgItem.Name;
            }
        }

        public string Location
        {
            get
            {
                return rgItem.Location;
            }
        }

        public void Compute()
        {
            ActivateItem(new AzureComputeViewModel(windowManager, eventAggregator, tenantId, subscriptionId, rgItem.Name, authType));
        }

        public void StorageAccount()
        {
            ActivateItem(new AzureStorageViewModel(windowManager, eventAggregator, tenantId, subscriptionId, rgItem.Name, authType));
        }

        public void Network()
        {
            ActivateItem(new AzureNetworkViewModel(windowManager, eventAggregator, tenantId, subscriptionId, rgItem.Name, authType));
        }

        public void SqlDatabase()
        {
            ActivateItem(new AzureSqlViewModel(windowManager, eventAggregator, tenantId, subscriptionId, rgItem.Name, authType));
        }

        public void Deployment()
        {
            ActivateItem(new DeploymentListViewModel(windowManager, eventAggregator, tenantId, subscriptionId, rgItem.Name, authType));
        }
    }
}
