﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure.DataModel.Storage;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.Azure.Storage
{
    [Export(typeof(StorageAccountKeysViewModel))]
    public class StorageAccountKeysViewModel : Screen
    {
        private readonly IWindowManager windowManager;
        private readonly IEventAggregator eventAggregator;
        private string tenantId;
        private string subscriptionId;
        private string resourceGroupName;
        private string storageAccountName;
        private LoginType authType;
        private readonly BackgroundWorker worker = new BackgroundWorker();
        private Visibility showLoadIndicator = Visibility.Collapsed;
        private StorageAccountKeys keys = null;
        private string key1;
        private string key2;

        [ImportingConstructor]
        public StorageAccountKeysViewModel(IWindowManager winMgr, IEventAggregator evt, string tId, string sId, string rgName, string saName, LoginType loginType)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            tenantId = tId;
            subscriptionId = sId;
            resourceGroupName = rgName;
            storageAccountName = saName;
            authType = loginType;

            worker.DoWork += Worker_DoWork;
            worker.RunWorkerCompleted += Worker_RunWorkerCompleted;
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }

        void Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ShowLoadIndicator = Visibility.Collapsed;
            LoadStorageAccountKeys();
        }

        void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            AuthTokenCacheInfo cacheInfo = null;
            if (authType == LoginType.Interactive)
            {
                Task<AuthTokenCacheInfo> cacheInfoTask = App.CurrentAppContext.AuthManager.GetToken(Constants.ClientId, tenantId, "https://management.azure.com/");
                cacheInfoTask.Wait();
                cacheInfo = cacheInfoTask.Result;
            }
            else if (authType == LoginType.Spn)
            {
                cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(tenantId, App.CurrentAppContext.Settings.ManagementAppId, App.CurrentAppContext.Settings.ManagementAppSecret, "https://management.azure.com/");
            }
            if (cacheInfo != null)
            {
                Task<CloudSolutionProviderManagement.Azure.DataModel.Storage.StorageAccountKeys> keyTask = AzureResourceManager.ListStorageAccountKeys(cacheInfo, handler, subscriptionId, resourceGroupName, storageAccountName);
                keyTask.Wait();
                keys = keyTask.Result;
            }
        }

        public Visibility ShowLoadIndicator
        {
            get
            {
                return showLoadIndicator;
            }
            set
            {
                showLoadIndicator = value;
                NotifyOfPropertyChange(() => ShowLoadIndicator);
            }
        }

        public void LoadStorageAccountKeys()
        {
            if (keys != null)
            {
                Key1 = keys.Key1;
                Key2 = keys.Key2;
                Refresh();
            }
        }

        public string Key1
        {
            get
            {
                return key1;
            }
            set
            {
                key1 = value;
                NotifyOfPropertyChange(() => Key1);
            }
        }

        public string Key2
        {
            get
            {
                return key2;
            }
            set
            {
                key2 = value;
                NotifyOfPropertyChange(() => Key2);
            }
        }

        public void Reload()
        {
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }
    }
}
