﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderManagement.Azure.DataModel.Storage;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.Azure.Storage
{
    [Export(typeof(StorageAccountViewModel))]
    public class StorageAccountViewModel : Conductor<object>
    {
        private readonly IWindowManager windowManager;
        private readonly IEventAggregator eventAggregator;
        private string tenantId;
        private string subscriptionId;
        private string resourceGroupName;
        private StorageAccount saItem;
        private LoginType authType;
        private readonly BackgroundWorker worker = new BackgroundWorker();

        [ImportingConstructor]
        public StorageAccountViewModel(IWindowManager winMgr, IEventAggregator evt, string tId, string sId, string rgName, StorageAccount item, LoginType loginType)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            tenantId = tId;
            subscriptionId = sId;
            resourceGroupName = rgName;
            saItem = item;
            authType = loginType;
        }

        public string StorageAccountId
        {
            get
            {
                return saItem.Id;
            }
        }


        public string StorageAccountName
        {
            get
            {
                return saItem.Name;
            }
        }

        public string Location
        {
            get
            {
                return saItem.Location;
            }
        }

        public string AccountType
        {
            get
            {
                return saItem.Properties.AccountType;
            }
        }

        public DateTime? CreationTime
        {
            get
            {
                return saItem.Properties.CreationTime;
            }
        }

        public string ProvisioningState
        {
            get
            {
                return saItem.Properties.ProvisioningState;
            }
        }

        public string PrimaryLocation
        {
            get
            {
                return saItem.Properties.PrimaryLocation;
            }
        }

        public List<Endpoints> PrimaryEndpoints
        {
            get
            {
                List<Endpoints> endpoints = new List<Endpoints>();
                endpoints.Add(saItem.Properties.PrimaryEndpoints);
                return endpoints; 
            }
        }

        public void Keys()
        {
            ActivateItem(new StorageAccountKeysViewModel(windowManager, eventAggregator, tenantId, subscriptionId, resourceGroupName, saItem.Name, authType));
        }
        
        
    }
}
