﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.Azure.VirtualMachine
{
    [Export(typeof(OSProfileViewModel))]
    public class OSProfileViewModel : Screen
    {
        private readonly IWindowManager windowManager;
        private readonly IEventAggregator eventAggregator;
        private string tenantId;
        private string subscriptionId;
        private string resourceGroupName;
        private CloudSolutionProviderManagement.Azure.DataModel.Compute.VirtualMachine vmItem;
        private LoginType authType;

        [ImportingConstructor]
        public OSProfileViewModel(IWindowManager winMgr, IEventAggregator evt, string tId, string sId, string rgName, CloudSolutionProviderManagement.Azure.DataModel.Compute.VirtualMachine item, LoginType loginType)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            tenantId = tId;
            subscriptionId = sId;
            resourceGroupName = rgName;
            vmItem = item;
            authType = loginType;
        }

        public string ComputerName
        {
            get
            {
                return vmItem.Properties.OSProfile.ComputerName;
            }
        }

        public string AdminUsername
        {
            get
            {
                return vmItem.Properties.OSProfile.AdminUsername;
            }
        }

        public string CustomData
        {
            get
            {
                return vmItem.Properties.OSProfile.CustomData;
            }
        }

        public string TimeZone
        {
            get
            {
                if (vmItem.Properties.OSProfile.WindowsConfiguration != null)
                    return vmItem.Properties.OSProfile.WindowsConfiguration.TimeZone;
                else
                    return null;
            }
        }

        public bool? EnableAutomaticUpdates
        {
            get
            {
                if (vmItem.Properties.OSProfile.WindowsConfiguration != null)
                    return vmItem.Properties.OSProfile.WindowsConfiguration.EnableAutomaticUpdates;
                else
                    return null;
            }
        }
    }
}
