﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderManagement.AzureActiveDirectory;
using Microsoft.Samples.CloudSolutionProviderManagement.AzureActiveDirectory.DataModel;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.AzureActiveDirectory.ServicePrincipal
{
    [Export(typeof(ServicePrincipalListViewModel))]
    public class ServicePrincipalListViewModel : Screen
    {
        private IWindowManager windowManager;
        private IEventAggregator eventAggregator;
        private ServicePrincipals spList = null;
        private ObservableCollection<ServicePrincipalViewModel> spModels = null;
        private readonly BackgroundWorker worker = new BackgroundWorker();
        private Visibility showLoadIndicator = Visibility.Collapsed;
        private string tenantguid = null;
        private LoginType authType;

        [ImportingConstructor]
        public ServicePrincipalListViewModel(IWindowManager winMgr, IEventAggregator evt, string tenantId, LoginType loginType)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            tenantguid = tenantId;
            authType = loginType;
            worker.DoWork += Worker_DoWork;
            worker.RunWorkerCompleted += Worker_RunWorkerCompleted;
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }

        void Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ShowLoadIndicator = Visibility.Collapsed;
            LoadServicePrincipals();
        }

        void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            AuthTokenCacheInfo cacheInfo = null;
            if (authType == LoginType.Interactive)
            {
                Task<AuthTokenCacheInfo> cacheInfoTask = App.CurrentAppContext.AuthManager.GetToken(App.CurrentAppContext.Settings.ClientId, tenantguid, "https://graph.windows.net");
                cacheInfoTask.Wait();
                cacheInfo = cacheInfoTask.Result;
            }
            else if (authType == LoginType.Spn)
            {
                cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(tenantguid, App.CurrentAppContext.Settings.ManagementAppId, App.CurrentAppContext.Settings.ManagementAppSecret, "https://graph.windows.net");
            }
            if (cacheInfo != null)
            {
                Task<ServicePrincipals> spListTask = AzureActiveDirectoryManager.GetServicePrincipals(cacheInfo, handler, tenantguid);
                spListTask.Wait();
                spList = spListTask.Result;
            }
        }

        public Visibility ShowLoadIndicator
        {
            get
            {
                return showLoadIndicator;
            }
            set
            {
                showLoadIndicator = value;
                NotifyOfPropertyChange(() => ShowLoadIndicator);
            }
        }

        public ObservableCollection<ServicePrincipalViewModel> ServicePrincipalModels
        {
            get
            {
                return spModels;
            }
            set
            {
                spModels = value;
                NotifyOfPropertyChange(() => ServicePrincipalModels);
            }
        }

        public void LoadServicePrincipals()
        {
            ServicePrincipalModels = new ObservableCollection<ServicePrincipalViewModel>();

            if (spList != null)
            {
                foreach (CloudSolutionProviderManagement.AzureActiveDirectory.DataModel.ServicePrincipal sp in spList.ServicePrincipalList)
                {
                    spModels.Add(new ServicePrincipalViewModel(windowManager, eventAggregator, tenantguid, sp, authType));
                }
            }
        }

        public void Reload()
        {
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }

        public void NewServicePrincipal()
        {
            if ((bool)windowManager.ShowDialog(new NewServicePrincipalViewModel(eventAggregator, tenantguid)))
            {
                Reload();
            }
        }
    }
}
