﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderManagement.AzureActiveDirectory;
using Microsoft.Samples.CloudSolutionProviderManagement.AzureActiveDirectory.DataModel;
using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.AzureActiveDirectory.ServicePrincipal
{
    [Export(typeof(ServicePrincipalViewModel))]
    public class ServicePrincipalViewModel : Screen
    {
        private readonly IWindowManager windowManager;
        private readonly IEventAggregator eventAggregator;
        private string tenantId;
        private LoginType authType;
        private CloudSolutionProviderManagement.AzureActiveDirectory.DataModel.ServicePrincipal servicePrincipal;

        [ImportingConstructor]
        public ServicePrincipalViewModel(IWindowManager winMgr, IEventAggregator evt, string tId, CloudSolutionProviderManagement.AzureActiveDirectory.DataModel.ServicePrincipal sp, LoginType loginType)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            tenantId = tId;
            servicePrincipal = sp;
            authType = loginType;
        }

        public string ServicePrincipalName
        {
            get
            {
                return servicePrincipal.ServicePrincipalName;
            }
        }

        public string ObjectId
        {
            get
            {
                return servicePrincipal.ObjectId;
            }
        }

        public string AppDisplayName
        {
            get
            {
                return servicePrincipal.AppDisplayName;
            }
        }

        public string AppId
        {
            get
            {
                return servicePrincipal.AppId;
            }
        }

        public string SPDisplayName
        {
            get
            {
                return servicePrincipal.DisplayName;
            }
        }

        public string PublisherName
        {
            get { return servicePrincipal.PublisherName; }
        }

        public async void AssignRole()
        {
            
            try
            {
                var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
                AuthTokenCacheInfo cacheInfo = null;
                if (authType == LoginType.Interactive)
                {
                    cacheInfo = await App.CurrentAppContext.AuthManager.GetToken(App.CurrentAppContext.Settings.ClientId, tenantId, "https://graph.windows.net");

                }
                else if (authType == LoginType.Spn)
                {
                    cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(tenantId, App.CurrentAppContext.Settings.ManagementAppId, App.CurrentAppContext.Settings.ManagementAppSecret, "https://graph.windows.net");
                }
                if (cacheInfo != null)
                {
                    CloudSolutionProviderManagement.AzureActiveDirectory.DataModel.DirectoryRoles roles = await AzureActiveDirectoryManager.GetDirectoryRoles(cacheInfo, handler, tenantId);
                    string adminRoleId = null;
                    foreach (DirectoryRole r in roles.DirectoryRoleList)
                    {
                        if (r.DisplayName == "Company Administrator")
                        {
                            adminRoleId = r.ObjectId;
                            break;
                        }
                    }
                    if (adminRoleId != null)
                    {
                        AddRoleMemberParameters p = new AddRoleMemberParameters();
                        p.Url = "https://graph.windows.net/" + tenantId + "/directoryObjects/" + servicePrincipal.ObjectId;
                        bool isSuccess=await AzureActiveDirectoryManager.AddDirectoryRoleMember(cacheInfo, handler, tenantId, adminRoleId, p);
                        if(isSuccess)
                        {
                            MessageBox.Show("Service Principal has been assigned to Admin Role", "Attention", MessageBoxButton.OK, MessageBoxImage.Information);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.ErrorLine(eventAggregator, "{0} {1}", ex.Message, ex.StackTrace);
                MessageBox.Show(ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }
    }
}
