﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderManagement.AzureActiveDirectory;
using Microsoft.Samples.CloudSolutionProviderManagement.AzureActiveDirectory.DataModel;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.AzureActiveDirectory.Tenant
{
    [Export(typeof(TenantViewModel))]
    public class TenantViewModel : Screen
    {
        private IWindowManager windowManager;
        private IEventAggregator eventAggregator;
        private string tenantId;
        private LoginType authType;
        private CloudSolutionProviderManagement.AzureActiveDirectory.DataModel.TenantDetail tenantDetail;
        private readonly BackgroundWorker worker = new BackgroundWorker();
        private Visibility showLoadIndicator = Visibility.Collapsed;

        [ImportingConstructor]
        public TenantViewModel(IWindowManager winMgr, IEventAggregator evt, string tId, LoginType loginType)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            tenantId = tId;
            authType = loginType;
            worker.DoWork += Worker_DoWork;
            worker.RunWorkerCompleted += Worker_RunWorkerCompleted;
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }

        void Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ShowLoadIndicator = Visibility.Collapsed;
            LoadTenantDetail();
        }

        void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            //get tenant detail info from AAD
            var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            AuthTokenCacheInfo cacheInfo = null;
            if (authType == LoginType.Interactive)
            {
                Task<AuthTokenCacheInfo> cacheInfoTask = App.CurrentAppContext.AuthManager.GetToken(App.CurrentAppContext.Settings.ClientId, tenantId, "https://graph.windows.net");
                cacheInfoTask.Wait();
                cacheInfo = cacheInfoTask.Result;
            }
            else if (authType == LoginType.Spn)
            {
                cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(tenantId, App.CurrentAppContext.Settings.ManagementAppId, App.CurrentAppContext.Settings.ManagementAppSecret, "https://graph.windows.net");
            }
            if (cacheInfo != null)
            {
                Task<CloudSolutionProviderManagement.AzureActiveDirectory.DataModel.TenantDetails> tenantListTask = AzureActiveDirectoryManager.GetTenantDetails(cacheInfo, handler, tenantId);
                tenantListTask.Wait();
                tenantDetail = tenantListTask.Result.TenantDetailList[0];
            }
        }

        public Visibility ShowLoadIndicator
        {
            get
            {
                return showLoadIndicator;
            }
            set
            {
                showLoadIndicator = value;
                NotifyOfPropertyChange(() => ShowLoadIndicator);
            }
        }

        public void LoadTenantDetail()
        {
            Refresh();
        }

        public string TenantDisplayName
        {
            get
            {
                if (tenantDetail != null)
                    return tenantDetail.DisplayName;
                else
                    return string.Empty;
            }
        }

        public string ObjectId
        {
            get
            {
                if (tenantDetail != null)
                    return tenantDetail.ObjectId;
                else
                    return string.Empty;
            }
        }

        public string TelephoneNumber
        {
            get
            {
                if (tenantDetail != null)
                    return tenantDetail.TelephoneNumber;
                else
                    return string.Empty;
            }
        }

        public AssignedPlan[] AssignedPlans
        {
            get
            {
                if (tenantDetail != null)
                    return tenantDetail.AssignedPlans;
                else
                    return null;
            }
        }

        public ProvisionedPlan[] ProvisionedPlans
        {
            get
            {
                if (tenantDetail != null)
                    return tenantDetail.ProvisionedPlans;
                else
                    return null;
            }
        }

        public VerifiedDomain[] VerifiedDomains
        {
            get
            {
                if (tenantDetail != null)
                    return tenantDetail.VerifiedDomains;
                else
                    return null;
            }
        }
    }
}
