﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderManagement.AzureActiveDirectory;
using Microsoft.Samples.CloudSolutionProviderManagement.AzureActiveDirectory.DataModel;
using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.AzureActiveDirectory.User
{
    [Export(typeof(NewUserViewModel))]
    public class NewUserViewModel : Screen
    {
        private string tenantId;
        IEventAggregator eventAggregator;
        private string userDisplayName;
        private string userPrincipalName;
        private string password;
        private string mailNickName;
        private LoginType authType;

        [ImportingConstructor]
        public NewUserViewModel(IEventAggregator evt, string tId, LoginType loginType)
        {
            eventAggregator = evt;
            tenantId = tId;
            authType = loginType;
            DisplayName = "Create New User";
        }

        public string UserDisplayName
        {
            get
            {
                return userDisplayName;
            }
            set
            {
                userDisplayName = value;
                NotifyOfPropertyChange(() => UserDisplayName);
            }
        }

        public string UserPrincipalName
        {
            get
            {
                return userPrincipalName;
            }
            set
            {
                userPrincipalName = value;
                NotifyOfPropertyChange(() => UserPrincipalName);
            }
        }

        public string MailNickName
        {
            get
            {
                return mailNickName;
            }
            set
            {
                mailNickName = value;
                NotifyOfPropertyChange(() => MailNickName);
            }
        }

        public string Password
        {
            get
            {
                return password;
            }
            set
            {
                password = value;
                NotifyOfPropertyChange(() => Password);
            }
        }

        public async void Save()
        {
            CreateUserParameters newUser = new CreateUserParameters();
            newUser.AccountEnabled = true;
            newUser.DisplayName = UserDisplayName;
            newUser.MailNickName = MailNickName;
            newUser.UserPrincipalName = UserPrincipalName;
            newUser.PasswordProfile = new PasswordProfile();
            newUser.PasswordProfile.ForcePasswordNextLogin = false;
            newUser.PasswordProfile.Password = Password;
            try
            {
                var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
                AuthTokenCacheInfo cacheInfo = null;
                if (authType == LoginType.Interactive)
                {
                    cacheInfo = await App.CurrentAppContext.AuthManager.GetToken(App.CurrentAppContext.Settings.ClientId, tenantId, "https://graph.windows.net");
                    
                }
                else if (authType == LoginType.Spn)
                {
                    cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(tenantId, App.CurrentAppContext.Settings.ManagementAppId, App.CurrentAppContext.Settings.ManagementAppSecret, "https://graph.windows.net");
                }
                if (cacheInfo != null)
                {
                    CloudSolutionProviderManagement.AzureActiveDirectory.DataModel.User user =await AzureActiveDirectoryManager.NewUser(cacheInfo, handler, tenantId, newUser);
                }
                TryClose(true);
            }
            catch(Exception ex)
            {
                Logger.ErrorLine(eventAggregator, "{0} {1}", ex.Message, ex.StackTrace);
                MessageBox.Show(ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        public void Cancel()
        {
            TryClose(false);
        }
    }
}
