﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderManagement.AzureActiveDirectory;
using Microsoft.Samples.CloudSolutionProviderManagement.AzureActiveDirectory.DataModel;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.AzureActiveDirectory.User
{
    [Export(typeof(UserListViewModel))]
    public class UserListViewModel : Screen
    {
        private IWindowManager windowManager;
        private IEventAggregator eventAggregator;
        private Users userList = null;
        private ObservableCollection<UserViewModel> users = null;
        private readonly BackgroundWorker worker = new BackgroundWorker();
        private Visibility showLoadIndicator = Visibility.Collapsed;
        private string tenantguid = null;
        private LoginType authType;

        [ImportingConstructor]
        public UserListViewModel(IWindowManager winManager, IEventAggregator evt, string tenantId, LoginType loginType)
        {
            windowManager = winManager;
            eventAggregator = evt;
            tenantguid = tenantId;
            authType = loginType;
            worker.DoWork += Worker_DoWork;
            worker.RunWorkerCompleted += Worker_RunWorkerCompleted;
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }

        void Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ShowLoadIndicator = Visibility.Collapsed;
            LoadUsers();
        }

        void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            AuthTokenCacheInfo cacheInfo=null;
            if (authType == LoginType.Interactive)
            {
                Task<AuthTokenCacheInfo> cacheInfoTask = App.CurrentAppContext.AuthManager.GetToken(App.CurrentAppContext.Settings.ClientId, tenantguid, "https://graph.windows.net");
                cacheInfoTask.Wait();
                cacheInfo = cacheInfoTask.Result;
            }
            else if(authType==LoginType.Spn)
            {
                cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(tenantguid, App.CurrentAppContext.Settings.ManagementAppId, App.CurrentAppContext.Settings.ManagementAppSecret, "https://graph.windows.net");
            }
            if (cacheInfo != null)
            {
                
                Task<Users> userListTask = AzureActiveDirectoryManager.GetUsers(cacheInfo, handler, tenantguid);
                userListTask.Wait();
                userList = userListTask.Result;
            }
        }

        public Visibility ShowLoadIndicator
        {
            get
            {
                return showLoadIndicator;
            }
            set
            {
                showLoadIndicator = value;
                NotifyOfPropertyChange(() => ShowLoadIndicator);
            }
        }

        public ObservableCollection<UserViewModel> Users
        {
            get
            {
                return users;
            }
            set
            {
                users = value;
                NotifyOfPropertyChange(() => Users);
            }
        }

        public void LoadUsers()
        {
            Users = new ObservableCollection<UserViewModel>();

            if (userList != null)
            {
                foreach (CloudSolutionProviderManagement.AzureActiveDirectory.DataModel.User user in userList.UserList)
                {
                    users.Add(new UserViewModel(windowManager, eventAggregator, tenantguid, user));
                }
            }
        }

        public void Reload()
        {
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }

        public void NewUser()
        {
            if ((bool)windowManager.ShowDialog(new NewUserViewModel(eventAggregator, tenantguid, authType)))
            {
                Reload();
            }
        }
    }
}
