﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.EventStream;
using System;
using System.ComponentModel.Composition;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels
{
    [Export(typeof(CSPAdminViewModel))]
    public class CSPAdminViewModel : Conductor<object>
    {
        private readonly IWindowManager windowMgr;
        private readonly IEventAggregator eventAggregator;

        [ImportingConstructor]
        public CSPAdminViewModel(IWindowManager winMgr, IEventAggregator evtAggregator)
        {
            windowMgr = winMgr;
            eventAggregator = evtAggregator;
            this.DisplayName = "Cloud Solution Provider Sample: Admin";
        }

        public bool IsLoggedIn
        {
            get
            {
                return CheckIsLogin();
            }            
        }

        public bool IsLoggedOut
        {
            get
            {
                return !CheckIsLogin();
            }
        }
        private bool CheckIsLogin()
        {
            if (App.CurrentAppContext.AuthManager != null
                && App.CurrentAppContext.AuthManager.IsCacheValid())
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public async void Login()
        {
            try {
                await App.CurrentAppContext.AuthManager.AcquireTokens(Constants.ClientId, Constants.AzureADTenantId, "https://graph.windows.net");

                Refresh();
            }
            catch(Exception ex)
            {
                Logger.ErrorLine(eventAggregator, "{0} {1}", ex.Message, ex.StackTrace);
            }
            
        }

        public void Logout()
        {
            App.CurrentAppContext.AuthManager.ClearTokenCache();
            NotifyOfPropertyChange(() => IsLoggedIn);
            NotifyOfPropertyChange(() => IsLoggedOut);
        }

        public void ResellerInfo()
        {
            ActivateItem(new ResellerViewModel(windowMgr, eventAggregator, LoginType.Interactive));
        }

        public void ListCustomers()
        {
            ActivateItem(new CustomerListViewModel(windowMgr, eventAggregator, LoginType.Interactive));
        }
        
    }
}
