﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.AzureActiveDirectory.Application;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.AzureActiveDirectory.Group;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.AzureActiveDirectory.ServicePrincipal;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.AzureActiveDirectory.Tenant;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.AzureActiveDirectory.User;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Profile;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter.DataModel;
using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels
{
    [Export(typeof(CSPCustomerViewModel))]
    public class CSPCustomerViewModel : Conductor<object>
    {
        private readonly IWindowManager windowMgr;
        private readonly IEventAggregator eventAggregator;
        private string customerTenantId;

        [ImportingConstructor]
        public CSPCustomerViewModel(IWindowManager winMgr, IEventAggregator evtAggregator)
        {
            windowMgr = winMgr;
            eventAggregator = evtAggregator;
            this.DisplayName = "Cloud Solution Provider Sample: Customer";
        }

        public bool IsLoggedIn
        {
            get
            {
                return CheckIsLogin();
            }
        }

        public bool IsLoggedOut
        {
            get
            {
                return !CheckIsLogin();
            }
        }
        private bool CheckIsLogin()
        {
            if (App.CurrentAppContext.AuthManager != null
                && App.CurrentAppContext.AuthManager.IsCacheValid())
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public async void Login()
        {
            try
            {
                await App.CurrentAppContext.AuthManager.AcquireTokens(Constants.ClientId, Constants.AzureADTenantId, "https://graph.windows.net");
                AuthTokenCache cache = App.CurrentAppContext.AuthManager.TokenStorage.GetCache();
                AuthTokenCacheInfo cacheInfo=null;
                foreach(AuthTokenCacheInfo cInfo in cache.GetValues("https://graph.windows.net"))
                {
                    cacheInfo = cInfo;
                    customerTenantId = cInfo.TenantId;
                }
                
                

                Refresh();
            }
            catch (Exception ex)
            {
                Logger.ErrorLine(eventAggregator, "{0} {1}", ex.Message, ex.StackTrace);
            }

        }

        public void Logout()
        {
            App.CurrentAppContext.AuthManager.ClearTokenCache();
            NotifyOfPropertyChange(() => IsLoggedIn);
            NotifyOfPropertyChange(() => IsLoggedOut);
        }

        public void TenantDetails()
        {
            ActivateItem(new TenantViewModel(windowMgr, eventAggregator, customerTenantId, LoginType.Interactive));
        }

        public void Users()
        {
            ActivateItem(new UserListViewModel(windowMgr, eventAggregator, customerTenantId, LoginType.Interactive));
        }

        public void Applications()
        {
            ActivateItem(new ApplicationListViewModel(windowMgr, eventAggregator, customerTenantId, LoginType.Interactive));
        }

        public void Groups()
        {
            ActivateItem(new GroupListViewModel(windowMgr, eventAggregator, customerTenantId, LoginType.Interactive));
        }

        public void ServicePrincipals()
        {
            ActivateItem(new ServicePrincipalListViewModel(windowMgr, eventAggregator, customerTenantId, LoginType.Interactive));
        }

    }
}
