﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.EventStream;
using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels
{
    [Export(typeof(CSPServerViewModel))]
    public class CSPServerViewModel : Conductor<object>
    {
        private readonly IWindowManager windowMgr;
        private readonly IEventAggregator eventAggregator;
        private AuthTokenCacheInfo cacheInfo = null;

        [ImportingConstructor]
        public CSPServerViewModel(IWindowManager winMgr, IEventAggregator evtAggregator)
        {
            windowMgr = winMgr;
            eventAggregator = evtAggregator;
            this.DisplayName = "Cloud Solution Provider Sample: Automation";
            Initialize();
        }

        protected override void OnViewLoaded(object view)
        {
            base.OnViewLoaded(view);
        }

        public bool IsLoggedIn
        {
            get
            {
                return cacheInfo!=null;
            }
        }

        public void Initialize()
        {

            try
            {
                cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(App.CurrentAppContext.Settings.ResellerTenantId, App.CurrentAppContext.Settings.ManagementAppId, App.CurrentAppContext.Settings.ManagementAppSecret, "https://graph.windows.net");
                
                Refresh();
            }
            catch (Exception ex)
            {
                Logger.ErrorLine(eventAggregator, "{0} {1}", ex.Message, ex.StackTrace);
            }
        }

        public void ResellerInfo()
        {
            ActivateItem(new ResellerViewModel(windowMgr, eventAggregator, LoginType.Spn));
        }

        public void ListCustomers()
        {
            ActivateItem(new CustomerListViewModel(windowMgr, eventAggregator, LoginType.Spn));
        }

        public void Events()
        {
            ActivateItem(new EventStreamViewModel(windowMgr, eventAggregator));
        }
    }
}
