﻿using Caliburn.Micro;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Models.Events;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Views;
using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Media;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels
{
    [Export(typeof(LogViewModel))]
    public class LogViewModel : Screen, IHandle<object>
    {
        public static readonly Brush DefaultBrush = Brushes.Black;
        public static readonly Brush InfoBrush = Brushes.LightGreen;
        public static readonly Brush WarnBrush = Brushes.Yellow;
        public static readonly Brush ErrorBrush = Brushes.Red;

        private RichTextBox logRTB;
        private static FlowDocument flowDoc;

        [ImportingConstructor]
        public LogViewModel(IEventAggregator eventAg)
        {
            this.DisplayName = "Logging Information";
            eventAg.Subscribe(this);
        }

        protected override void OnViewLoaded(object view)
        {
            LogView logView = (LogView)view;
            logRTB = logView.LoggingRTB;
            flowDoc = logRTB.Document;
            base.OnViewLoaded(view);
        }

        public void ClearLog()
        {
            Application.Current.Dispatcher.Invoke(() =>
            {
                logRTB.Document.Blocks.Clear();
                logRTB.ScrollToEnd();
            });
        }

        public void Handle(object message)
        {
            if(message is TraceEvent)
            {
                TraceEvent traceEvent = (TraceEvent)message;
                Brush background=DefaultBrush;
                switch (traceEvent.EventType)
                {
                    case TraceEventType.Error:
                        background = ErrorBrush;
                        break;
                    case TraceEventType.Warning:
                        background = WarnBrush;
                        break;
                    case TraceEventType.Info:
                        background = InfoBrush;
                        break;
                    default:
                        background = DefaultBrush;
                        break;
                }
                Application.Current.Dispatcher.Invoke(() =>
                {
                    var para = new Paragraph(new Run(traceEvent.Content));
                    para.Background = background;
                    logRTB.Document.Blocks.Add(para);
                    logRTB.ScrollToEnd();
                });
            }
        }
    }
}
