﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Customer
{
    [Export(typeof(CustomerListItemViewModel))]
    public class CustomerListItemViewModel : Screen
    {
        private readonly IWindowManager windowManager;
        private readonly IEventAggregator eventAggregator;
        private string customerTenantId;
        private string customerTenantName;
        private string customerDefaultDomainName;
        private LoginType authType;

        [ImportingConstructor]
        public CustomerListItemViewModel(IWindowManager winMgr, IEventAggregator evt, string tenantName, string tenantId,  string defaultDomainName, LoginType loginType)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            customerTenantId = tenantId;
            customerTenantName = tenantName;
            customerDefaultDomainName = defaultDomainName;
            authType = loginType;
        }

        public string CustomerName
        {
            get
            {
                return customerTenantName;
            }
        }

        public string CustomerTenantId
        {
            get
            {
                return customerTenantId;
            }
        }

        public string CustomerDefaultDomainName
        {
            get
            {
                return customerDefaultDomainName;
            }
        }

        public async void Manage()
        {
            AuthTokenCacheInfo token = null;
            if (authType == LoginType.Interactive)
            {
                await App.CurrentAppContext.AuthManager.AcquireTokens(Constants.ClientId, customerTenantId, "https://graph.windows.net/");
                token = await App.CurrentAppContext.AuthManager.GetToken(Constants.ClientId, customerTenantId, "https://graph.windows.net/");
            }
            else if(authType==LoginType.Spn)
            {
                //token = await App.CurrentAppContext.AuthManager.GetToken(App.CurrentAppContext.Settings.ManagementAppId, App.CurrentAppContext.Settings.ResellerTenantId, "https://graph.windows.net/");
                token = App.CurrentAppContext.AuthManager.GetTokenBySpn(customerTenantId, App.CurrentAppContext.Settings.ManagementAppId, App.CurrentAppContext.Settings.ManagementAppSecret, "https://graph.windows.net/");
            }
            if(token!= null)
                windowManager.ShowWindow(new CustomerViewModel(windowManager, eventAggregator, customerTenantName, customerTenantId, customerDefaultDomainName, App.CurrentAppContext.Settings.ResellerTenantId, authType), null, null);
        }
    }
}
