﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.AzureActiveDirectory.Application;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.AzureActiveDirectory.Group;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.AzureActiveDirectory.ServicePrincipal;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.AzureActiveDirectory.Tenant;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.AzureActiveDirectory.User;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Entitlement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Order;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Profile;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Subscription;
using Microsoft.Samples.CloudSolutionProviderManagement.AzureActiveDirectory;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter.DataModel;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Customer
{
    [Export(typeof(CustomerViewModel))]
    public class CustomerViewModel : Conductor<object>
    {
        private readonly IWindowManager windowManager;
        private readonly IEventAggregator eventAggregator;
        private string customerTenantName;
        private string customerTenantId;
        private string customerDefaultDomainName;
        private string customerCid;
        private string resellerCid;
        private LoginType authType;
        private readonly BackgroundWorker worker = new BackgroundWorker();
        private Visibility showLoadIndicator = Visibility.Collapsed;

        [ImportingConstructor]
        public CustomerViewModel(IWindowManager winMgr, IEventAggregator evt, string name, string tenantId, string defaultDomainName, string resellerId, LoginType loginType)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            customerDefaultDomainName = defaultDomainName;
            customerTenantId = tenantId;
            customerTenantName = name;
            resellerCid = resellerId;
            authType = loginType;
            this.DisplayName = "Manage Customer: " + customerTenantName;
            worker.RunWorkerCompleted += worker_RunWorkerCompleted;
            worker.DoWork += worker_DoWork;
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }

        protected override void OnViewLoaded(object view)
        {
            base.OnViewLoaded(view);
        }

        void worker_DoWork(object sender, DoWorkEventArgs e)
        {
            GetCustomerInfo();
        }

        void worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ShowLoadIndicator = Visibility.Collapsed;
        }

        public Visibility ShowLoadIndicator
        {
            get
            {
                return showLoadIndicator;
            }
            set
            {
                showLoadIndicator = value;
                NotifyOfPropertyChange(() => ShowLoadIndicator);
            }
        }

        private void GetCustomerInfo()
        {
            var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            AuthTokenCacheInfo cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(App.CurrentAppContext.Settings.ResellerTenantId, App.CurrentAppContext.Settings.PartnerCenterAppId, App.CurrentAppContext.Settings.PartnerCenterAppSecret, Microsoft.Samples.CloudSolutionProviderManagement.Constants.AzureADGraphAPIUrl);
            PartnerCenterAuthToken authToken = new PartnerCenterAuthToken(cacheInfo.AccessToken, cacheInfo.ExpiresOn);

            Task<PartnerCenterAuthToken> saTokenTask = PartnerCenterManager.GetSalesAgentToken(authToken, handler);
            saTokenTask.Wait();
            handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            Task<CloudSolutionProviderManagement.PartnerCenter.DataModel.Customer> customerTask = PartnerCenterManager.GetCustomer(saTokenTask.Result, handler, customerTenantId, App.CurrentAppContext.Settings.ResellerTenantId);
            customerTask.Wait();
            customerCid = customerTask.Result.Id;
            
        }

        public void Profile()
        {
            ActivateItem(new ProfileListViewModel(windowManager, eventAggregator, customerTenantId, customerCid));
        }

        public void User()
        {
            ActivateItem(new UserListViewModel(windowManager, eventAggregator, customerTenantId, authType));
        }

        public void Group()
        {
            ActivateItem(new GroupListViewModel(windowManager, eventAggregator, customerTenantId, authType));
        }

        public void Application()
        {
            ActivateItem(new ApplicationListViewModel(windowManager, eventAggregator, customerTenantId, authType));
        }

        public void ServicePrincipal()
        {
            ActivateItem(new ServicePrincipalListViewModel(windowManager, eventAggregator, customerTenantId, authType));
        }

        public void TenantDetail()
        {
            ActivateItem(new TenantViewModel(windowManager, eventAggregator, customerTenantId, authType));
        }

        public void Subscription()
        {
            ActivateItem(new SubscriptionListViewModel(windowManager, eventAggregator, customerTenantId, resellerCid, customerCid, authType));
        }

        public void Order()
        {
            ActivateItem(new OrderListViewModel(windowManager, eventAggregator, customerTenantId, resellerCid, customerCid));
        }

        public void Entitlement()
        {
            ActivateItem(new EntitlementListViewModel(windowManager, eventAggregator, customerTenantId, resellerCid, customerCid));
        }
    }
}
