﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Controls.Validation;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter.DataModel;
using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Customer
{
    [Export(typeof(NewCustomerViewModel))]
    public class NewCustomerViewModel : ValidatingScreen<NewCustomerViewModel>
    {
        private readonly IEventAggregator eventAggregator;
        private string companyName;
        private string primaryDomainPrefix;
        private string address1;
        private string address2;
        private string city;
        private string state;
        private string zipCode;
        private string country;
        private string firstName;
        private string lastName;
        private string email;
        private string phoneNumber;
        private string adminUserName;
        private string adminUserPassword;

        [ImportingConstructor]
        public NewCustomerViewModel(IEventAggregator evt)
        {
            eventAggregator = evt;
            DisplayName = "Create New Customer";
        }

        [Required]
        public string CompanyName
        {
            get
            {
                return companyName;
            }
            set
            {
                companyName = value;
                NotifyOfPropertyChange(() => CompanyName);
            }
        }

        [Required]
        public string PrimaryDomainPrefix
        {
            get
            {
                return primaryDomainPrefix;
            }
            set
            {
                primaryDomainPrefix = value;
                NotifyOfPropertyChange(() => PrimaryDomainPrefix);
            }
        }
        [Required]
        public string Address1
        {
            get
            {
                return address1;
            }
            set
            {
                address1 = value;
                NotifyOfPropertyChange(() => Address1);
            }
        }

        public string Address2
        {
            get
            {
                return address2;
            }
            set
            {
                address2 = value;
                NotifyOfPropertyChange(() => Address2);
            }
        }

        [Required]
        public string City
        {
            get
            {
                return city;
            }
            set
            {
                city = value;
                NotifyOfPropertyChange(() => City);
            }
        }

        [Required]
        public string State
        {
            get
            {
                return state;
            }
            set
            {
                state = value;
                NotifyOfPropertyChange(() => State);
            }
        }

        [Required]
        public string ZipCode
        {
            get
            {
                return zipCode;
            }
            set
            {
                zipCode = value;
                NotifyOfPropertyChange(() => ZipCode);
            }
        }
        [Required]
        public string Country
        {
            get
            {
                return country;
            }
            set
            {
                country = value;
                NotifyOfPropertyChange(() => Country);
            }
        }

        [Required]
        public string FirstName
        {
            get
            {
                return firstName;
            }
            set
            {
                firstName = value;
                NotifyOfPropertyChange(() => FirstName);
            }
        }
        [Required]
        public string LastName
        {
            get
            {
                return lastName;
            }
            set
            {
                lastName = value;
                NotifyOfPropertyChange(() => LastName);
            }
        }

        [Required]
        public string Email
        {
            get
            {
                return email;
            }
            set
            {
                email = value;
                NotifyOfPropertyChange(() => Email);
            }
        }

        [Required]
        public string PhoneNumber
        {
            get
            {
                return phoneNumber;
            }
            set
            {
                phoneNumber = value;
                NotifyOfPropertyChange(() => PhoneNumber);
            }
        }

        [Required]
        public string AdminUserName
        {
            get
            {
                return adminUserName;
            }
            set
            {
                adminUserName = value;
                NotifyOfPropertyChange(() => AdminUserName);
            }
        }

        [Required]
        public string AdminUserPassword
        {
            get
            {
                return adminUserPassword;
            }
            set
            {
                adminUserPassword = value;
                NotifyOfPropertyChange(() => AdminUserPassword);
            }
        }

        public async void Save()
        {
            NewCustomer newCustomer = new NewCustomer();
            newCustomer.DomainPrefix = PrimaryDomainPrefix;
            newCustomer.Password = AdminUserPassword;
            newCustomer.UserName = AdminUserName;
            newCustomer.Profile = new NewCustomerProfile();
            newCustomer.Profile.CompanyName = CompanyName;
            newCustomer.Profile.Culture = "en-US";
            newCustomer.Profile.Email = Email;
            newCustomer.Profile.FirstName = FirstName;
            newCustomer.Profile.Language = "en";
            newCustomer.Profile.LastName = LastName;
            newCustomer.Profile.Type = "organization";
            newCustomer.Profile.DefaultAddress = new NewCustomerDefaultAddress();
            newCustomer.Profile.DefaultAddress.FirstName = FirstName;
            newCustomer.Profile.DefaultAddress.LastName = LastName;
            newCustomer.Profile.DefaultAddress.AddressLine1 = Address1;
            newCustomer.Profile.DefaultAddress.AddressLine2 = Address2;
            newCustomer.Profile.DefaultAddress.City = City;
            newCustomer.Profile.DefaultAddress.Region = State;
            newCustomer.Profile.DefaultAddress.Country = Country;
            newCustomer.Profile.DefaultAddress.PostalCode = ZipCode;
            newCustomer.Profile.DefaultAddress.PhoneNumber = PhoneNumber;

            try {
                var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
                AuthTokenCacheInfo cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(App.CurrentAppContext.Settings.ResellerTenantId, App.CurrentAppContext.Settings.PartnerCenterAppId, App.CurrentAppContext.Settings.PartnerCenterAppSecret, Microsoft.Samples.CloudSolutionProviderManagement.Constants.AzureADGraphAPIUrl);
                PartnerCenterAuthToken authToken = new PartnerCenterAuthToken(cacheInfo.AccessToken, cacheInfo.ExpiresOn);

                PartnerCenterAuthToken saToken = await PartnerCenterManager.GetSalesAgentToken(authToken, handler);
                
                handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
                CloudSolutionProviderManagement.PartnerCenter.DataModel.Customer customer  = await PartnerCenterManager.NewCustomer(saToken, handler, App.CurrentAppContext.Settings.ResellerTenantId, newCustomer);
                TryClose(true);
            }
            catch(Exception ex)
            {
                Logger.ErrorLine(eventAggregator, "{0} {1}", ex.Message, ex.StackTrace);
                MessageBox.Show(ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        public void Cancel()
        {
            TryClose(false);
        }
    }
}
