﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Customer;
using Microsoft.Samples.CloudSolutionProviderManagement.AzureActiveDirectory;
using Microsoft.Samples.CloudSolutionProviderManagement.AzureActiveDirectory.DataModel;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter
{
    [Export(typeof(CustomerListViewModel))]
    public class CustomerListViewModel : Screen
    {
        private IWindowManager windowManager;
        private IEventAggregator eventAggregator;
        private Contract contractedTenants = null;
        private ObservableCollection<CustomerListItemViewModel> customers = null;
        private readonly BackgroundWorker worker = new BackgroundWorker();
        private Visibility showLoadIndicator = Visibility.Collapsed;
        private LoginType authType;

        [ImportingConstructor]
        public CustomerListViewModel(IWindowManager winManager, IEventAggregator evt, LoginType loginType)
        {
            windowManager = winManager;
            eventAggregator = evt;
            authType = loginType;
            worker.DoWork += Worker_DoWork;
            worker.RunWorkerCompleted += Worker_RunWorkerCompleted;
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }

        void Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ShowLoadIndicator = Visibility.Collapsed;
            LoadCustomers();
        }

        void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            AuthTokenCacheInfo cacheInfo = null;
            if (authType == LoginType.Interactive)
            {
                Task<AuthTokenCacheInfo> cacheInfoTask = App.CurrentAppContext.AuthManager.GetToken(App.CurrentAppContext.Settings.ClientId, App.CurrentAppContext.Settings.ResellerTenantId, "https://graph.windows.net");
                cacheInfoTask.Wait();
                cacheInfo = cacheInfoTask.Result;
            }
            else if(authType==LoginType.Spn)
            {
                cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(App.CurrentAppContext.Settings.ResellerTenantId, App.CurrentAppContext.Settings.ManagementAppId, App.CurrentAppContext.Settings.ManagementAppSecret, "https://graph.windows.net");
            }
            if (cacheInfo != null)
            {
                Task<Contract> contractTask = AzureActiveDirectoryManager.GetCustomers(cacheInfo, handler);
                contractTask.Wait();
                contractedTenants = contractTask.Result;
            }
        }

        public Visibility ShowLoadIndicator
        {
            get
            {
                return showLoadIndicator;
            }
            set
            {
                showLoadIndicator = value;
                NotifyOfPropertyChange(() => ShowLoadIndicator);
            }
        }

        public ObservableCollection<CustomerListItemViewModel> Customers
        {
            get
            {
                return customers;
            }
            set
            {
                customers = value;
                NotifyOfPropertyChange(() => Customers);
            }
        }

        public void LoadCustomers()
        {
            Customers = new ObservableCollection<CustomerListItemViewModel>();

            if (contractedTenants != null)
            {
                foreach (ContractValue cValue in contractedTenants.Value)
                {
                    Customers.Add(new CustomerListItemViewModel(windowManager, eventAggregator, cValue.DisplayName, cValue.CustomerContextId, cValue.DefaultDomainName, authType));
                }
            }
        }

        public void Reload()
        {
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }

        public void NewCustomer()
        {
            if ((bool)windowManager.ShowDialog(new NewCustomerViewModel(eventAggregator)))
            {
                Reload();
            }
        }
    }
}
