﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter.DataModel;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Entitlement
{
    [Export(typeof(EntitlementListViewModel))]
    public class EntitlementListViewModel : Screen
    {

        private IWindowManager windowManager;
        private IEventAggregator eventAggregator;
        private Entitlements entitlements = null;
        private ObservableCollection<EntitlementViewModel> entitlementModels = null;
        private readonly BackgroundWorker worker = new BackgroundWorker();
        private Visibility showLoadIndicator = Visibility.Collapsed;
        private string customerId = null;
        private string rcid = null;
        private string customerTenantId = null;

        [ImportingConstructor]
        public EntitlementListViewModel(IWindowManager winMgr, IEventAggregator evt, string tenantId, string resellerCid, string customerCid)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            customerId = customerCid;
            rcid = resellerCid;
            customerTenantId = tenantId;
            worker.DoWork += Worker_DoWork;
            worker.RunWorkerCompleted += Worker_RunWorkerCompleted;
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }

        public Visibility ShowLoadIndicator
        {
            get
            {
                return showLoadIndicator;
            }
            set
            {
                showLoadIndicator = value;
                NotifyOfPropertyChange(() => ShowLoadIndicator);
            }
        }

        private void Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ShowLoadIndicator = Visibility.Collapsed;
            LoadEntitlements();
        }

        private void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            AuthTokenCacheInfo cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(App.CurrentAppContext.Settings.ResellerTenantId, App.CurrentAppContext.Settings.PartnerCenterAppId, App.CurrentAppContext.Settings.PartnerCenterAppSecret, Microsoft.Samples.CloudSolutionProviderManagement.Constants.AzureADGraphAPIUrl);
            PartnerCenterAuthToken authToken = new PartnerCenterAuthToken(cacheInfo.AccessToken, cacheInfo.ExpiresOn);

            Task<PartnerCenterAuthToken> caTokenTask = PartnerCenterManager.GetCustomerAgentToken(authToken, customerId, handler);
            caTokenTask.Wait();
            handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            Task<Entitlements> entitlementTask = PartnerCenterManager.GetEntitlements(caTokenTask.Result, handler, customerId);
            entitlementTask.Wait();
            entitlements = entitlementTask.Result;
        }

        public ObservableCollection<EntitlementViewModel> EntitlementModels
        {
            get
            {
                return entitlementModels;
            }
            set
            {
                entitlementModels = value;
                NotifyOfPropertyChange(() => EntitlementModels);
            }
        }

        public void LoadEntitlements()
        {
            EntitlementModels = new ObservableCollection<EntitlementViewModel>();

            if (entitlements != null)
            {
                foreach (EntitlementItem item in entitlements.Items)
                {
                    EntitlementModels.Add(new EntitlementViewModel(windowManager, eventAggregator, rcid, item));
                }
            }
        }
    }
}
