﻿using Caliburn.Micro;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter.DataModel;
using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Entitlement
{
    [Export(typeof(EntitlementViewModel))]
    public class EntitlementViewModel : Screen
    {
        private readonly IWindowManager windowManager;
        private readonly IEventAggregator eventAggregator;
        private EntitlementItem entitlementItem;
        private string resellerCid;

        [ImportingConstructor]
        public EntitlementViewModel(IWindowManager winMgr, IEventAggregator evt, string rcid, EntitlementItem item)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            entitlementItem = item;
            resellerCid = rcid;
        }

        public string EntitlementId
        {
            get
            {
                return entitlementItem.Id;
            }
        }

        public string Product
        {
            get
            {
                return entitlementItem.ProductUri;
            }
        }

        public int Quantity
        {
            get
            {
                return entitlementItem.Quantity;
            }
        }

        public string StartDate
        {
            get
            {
                return entitlementItem.StartDate;
            }
        }

        public string EndDate
        {
            get
            {
                return entitlementItem.EndDate;
            }
        }

        public bool Trial
        {
            get
            {
                return entitlementItem.Trial;
            }
        }

        public string State
        {
            get
            {
                return entitlementItem.State;
            }
        }

        public string SuspensionReasons
        {
            get
            {
                string reasons = string.Empty;
                foreach (object obj in entitlementItem.SuspensionReasons)
                {
                    reasons += obj.ToString() + ";";
                }
                return reasons;
            }
        }

        public string FriendlyName
        {
            get
            {
                return entitlementItem.FriendlyName == null ? string.Empty : entitlementItem.FriendlyName.ToString();
            }
        }

        public void Usage()
        {
            windowManager.ShowDialog(new UsageViewModel(windowManager, eventAggregator, resellerCid, EntitlementId));
        }
    }
}
