﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter.DataModel;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Entitlement
{
    [Export(typeof(UsageViewModel))]
    public class UsageViewModel : Screen
    {
        private IWindowManager windowManager;
        private IEventAggregator eventAggregator;
        private string rcid = null;
        private string entitlementId;
        private DateTime startDate=DateTime.Now.AddDays(-1);
        private DateTime endDate=DateTime.Now;
        private List<string> granularity;
        private bool showDetails;
        private int count = 1000;
        private UsageRecordItem[] items;
        private readonly BackgroundWorker worker = new BackgroundWorker();
        private Visibility showLoadIndicator = Visibility.Collapsed;

        [ImportingConstructor]
        public UsageViewModel(IWindowManager winMgr, IEventAggregator evt, string resellerCid, string eId)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            rcid = resellerCid;
            entitlementId = eId;
            DisplayName = "Usage Records";
            granularity = new List<string>();
            granularity.Add("Daily");
            granularity.Add("Hourly");
            worker.DoWork += Worker_DoWork;
            worker.RunWorkerCompleted += Worker_RunWorkerCompleted;
            
        }

        void Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ShowLoadIndicator = Visibility.Collapsed;
            LoadUsageRecords();
        }

        void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            string queryOption = "&reported_start_time="+StartDate.ToString("u")+ "&reported_end_time="+EndDate.ToString("u");
            var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            AuthTokenCacheInfo cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(App.CurrentAppContext.Settings.ResellerTenantId, App.CurrentAppContext.Settings.PartnerCenterAppId, App.CurrentAppContext.Settings.PartnerCenterAppSecret, Microsoft.Samples.CloudSolutionProviderManagement.Constants.AzureADGraphAPIUrl);
            PartnerCenterAuthToken authToken = new PartnerCenterAuthToken(cacheInfo.AccessToken, cacheInfo.ExpiresOn);

            Task<PartnerCenterAuthToken> saTokenTask = PartnerCenterManager.GetSalesAgentToken(authToken, handler);
            saTokenTask.Wait();
            handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            Task<UsageRecord> usageTask = PartnerCenterManager.GetusageRecords(saTokenTask.Result, handler, rcid, entitlementId, queryOption);
            usageTask.Wait();
            items = usageTask.Result.Items;
        }

        public void LoadUsageRecords()
        {
            Refresh();
        }

        public UsageRecordItem[] UsageRecords
        {
            get
            {
                return items;
            }
        }

        public Visibility ShowLoadIndicator
        {
            get
            {
                return showLoadIndicator;
            }
            set
            {
                showLoadIndicator = value;
                NotifyOfPropertyChange(() => ShowLoadIndicator);
            }
        }

        public DateTime StartDate
        {
            get
            {
                return startDate;
            }
            set
            {
                startDate = value;
                NotifyOfPropertyChange(() => StartDate);
            }
        }

        public DateTime EndDate
        {
            get
            {
                return endDate;
            }
            set
            {
                endDate = value;
                NotifyOfPropertyChange(() => EndDate);
            }
        }

        public List<string> Granularity
        {
            get
            {
                return granularity;
            }
            set
            {
                granularity = value;
                NotifyOfPropertyChange(() => Granularity);
            }
        }

        public bool ShowDetails
        {
            get
            {
                return showDetails;
            }
            set
            {
                showDetails = value;
                NotifyOfPropertyChange(() => ShowDetails);
            }
        }

        public int Count
        {
            get
            {
                return count;
            }
            set
            {
                count = value;
                NotifyOfPropertyChange(() => Count);
            }
        }

        public void GetUsageRecords()
        {
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }
    }
}
