﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter.DataModel;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Order
{
    [Export(typeof(NewOrderViewModel))]
    public class NewOrderViewModel : Screen
    {
        private readonly IEventAggregator eventAggregator;
        private string customerCid;
        ObservableCollection<OfferViewModel> offers = new ObservableCollection<OfferViewModel>();

        [ImportingConstructor]
        public NewOrderViewModel(IEventAggregator evt, string customerId)
        {
            eventAggregator = evt;
            customerCid = customerId;
            DisplayName = "New Order";
            LoadOffers();
        }

        public ObservableCollection<OfferViewModel> Offers
        {
            get
            {
                return offers;
            }
        }

        public void LoadOffers()
        {
            foreach(OfferGroup group in OfferCatalog.Catalog.OfferGroupCollection)
            {
                foreach(Offer o in group.Offers)
                {
                    offers.Add(new OfferViewModel(group.OfferCategory, o.Name, o.Id, o.Uri));
                }
            }
        }

        public async void Save()
        {
            NewOrder officeOrder = new NewOrder();
            List<NewUpdateOrderLineItem> officeItems = new List<NewUpdateOrderLineItem>();
            NewOrder azureOrder = new NewOrder();
            List<NewUpdateOrderLineItem> azureItems = new List<NewUpdateOrderLineItem>();
            int counter = 0;
            foreach(OfferViewModel offerModel in Offers)
            {
                if(offerModel.IsSelected)
                {
                    if (offerModel.Category == OfferCategory.Azure)
                    {
                        NewUpdateOrderLineItem lineItem = new NewUpdateOrderLineItem();
                        lineItem.LineItemNumber = counter;
                        lineItem.OfferUri = offerModel.OfferUri;
                        lineItem.Quantity = offerModel.Quantity;
                        azureItems.Add(lineItem);
                    }
                    else
                    {
                        NewUpdateOrderLineItem lineItem = new NewUpdateOrderLineItem();
                        lineItem.LineItemNumber = counter;
                        lineItem.OfferUri = offerModel.OfferUri;
                        lineItem.Quantity = offerModel.Quantity;
                        officeItems.Add(lineItem);
                        counter++;
                    }                    
                }
            }

            if (officeItems.Count>0||azureItems.Count>0)
            {
                
                try
                {
                    var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
                    AuthTokenCacheInfo cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(App.CurrentAppContext.Settings.ResellerTenantId, App.CurrentAppContext.Settings.PartnerCenterAppId, App.CurrentAppContext.Settings.PartnerCenterAppSecret, Microsoft.Samples.CloudSolutionProviderManagement.Constants.AzureADGraphAPIUrl);
                    PartnerCenterAuthToken authToken = new PartnerCenterAuthToken(cacheInfo.AccessToken, cacheInfo.ExpiresOn);

                    PartnerCenterAuthToken saToken = await PartnerCenterManager.GetSalesAgentToken(authToken, handler);
                    if (azureItems.Count > 0)
                    {
                        azureOrder.LineItems = azureItems.ToArray();
                        azureOrder.RecipientCustomerId = customerCid;
                        handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
                        CloudSolutionProviderManagement.PartnerCenter.DataModel.OrderItem order = await PartnerCenterManager.NewOrder(saToken, handler, App.CurrentAppContext.Settings.ResellerTenantId, azureOrder);
                    }
                    if(officeItems.Count>0)
                    {
                        officeOrder.LineItems = officeItems.ToArray();
                        officeOrder.RecipientCustomerId = customerCid;
                        handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
                        CloudSolutionProviderManagement.PartnerCenter.DataModel.OrderItem order = await PartnerCenterManager.NewOrder(saToken, handler, App.CurrentAppContext.Settings.ResellerTenantId, officeOrder);
                    }
                    TryClose(true);
                }
                catch (Exception ex)
                {
                    Logger.ErrorLine(eventAggregator, "{0} {1}", ex.Message, ex.StackTrace);
                    MessageBox.Show(ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
        }

        public void Cancel()
        {
            TryClose(false);
        }
    }
}
