﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter.DataModel;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Order
{
    [Export(typeof(OrderListViewModel))]
    public class OrderListViewModel : Screen
    {
        private IWindowManager windowManager;
        private IEventAggregator eventAggregator;
        private Orders orders = null;
        private ObservableCollection<OrderViewModel> orderModels = null;
        private readonly BackgroundWorker worker = new BackgroundWorker();
        private Visibility showLoadIndicator = Visibility.Collapsed;
        private string customerId = null;
        private string rcid = null;
        private string customerTenantId = null;

        [ImportingConstructor]
        public OrderListViewModel(IWindowManager winMgr, IEventAggregator evt, string tenantId, string resellerCid, string customerCid)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            customerId = customerCid;
            rcid = resellerCid;
            customerTenantId = tenantId;
            worker.DoWork += Worker_DoWork;
            worker.RunWorkerCompleted += Worker_RunWorkerCompleted;
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }

        public Visibility ShowLoadIndicator
        {
            get
            {
                return showLoadIndicator;
            }
            set
            {
                showLoadIndicator = value;
                NotifyOfPropertyChange(() => ShowLoadIndicator);
            }
        }

        private void Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ShowLoadIndicator = Visibility.Collapsed;
            LoadOrders();
        }

        private void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            AuthTokenCacheInfo cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(App.CurrentAppContext.Settings.ResellerTenantId, App.CurrentAppContext.Settings.PartnerCenterAppId, App.CurrentAppContext.Settings.PartnerCenterAppSecret, Microsoft.Samples.CloudSolutionProviderManagement.Constants.AzureADGraphAPIUrl);
            PartnerCenterAuthToken authToken = new PartnerCenterAuthToken(cacheInfo.AccessToken, cacheInfo.ExpiresOn);

            Task<PartnerCenterAuthToken> saTokenTask = PartnerCenterManager.GetSalesAgentToken(authToken, handler);
            saTokenTask.Wait();
            handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            Task<Orders> orderTask = PartnerCenterManager.GetCustomerOrders(saTokenTask.Result, handler, rcid, customerId);
            orderTask.Wait();
            orders = orderTask.Result;
        }

        public ObservableCollection<OrderViewModel> OrderModels
        {
            get
            {
                return orderModels;
            }
            set
            {
                orderModels = value;
                NotifyOfPropertyChange(() => OrderModels);
            }
        }

        public void LoadOrders()
        {
            OrderModels = new ObservableCollection<OrderViewModel>();

            if (orders != null)
            {
                foreach (OrderItem item in orders.Items)
                {
                    OrderModels.Add(new OrderViewModel(windowManager, eventAggregator, item));
                }
            }
        }

        public void Reload()
        {
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }

        public void NewOrder()
        {
            if((bool)windowManager.ShowDialog(new NewOrderViewModel(eventAggregator, customerId)))
            {
                Reload();
            }
        }
    }
}
