﻿using Caliburn.Micro;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter.DataModel;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Order
{
    [Export(typeof(OrderViewModel))]
    public class OrderViewModel : Screen
    {
        private readonly IWindowManager windowManager;
        private readonly IEventAggregator eventAggregator;
        private OrderItem orderItem;
        private ObservableCollection<OrderLineItemViewModel> orderLineitems;

        [ImportingConstructor]
        public OrderViewModel(IWindowManager winMgr, IEventAggregator evt, OrderItem item)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            orderItem = item;
            LoadLineItems();
        }

        public string OrderId
        {
            get
            {
                return orderItem.Id;
            }
        }

        public string CustomerId
        {
            get
            {
                return orderItem.RecipientCustomerId;
            }
        }

        public string ETag
        {
            get
            {
                return orderItem.Etag;
            }
        }

        public string CreationDate
        {
            get
            {
                return orderItem.CreationDate;
            }
        }

        public ObservableCollection<OrderLineItemViewModel> OrderLineItems
        {
            get
            {
                return orderLineitems;
            }
            set
            {
                orderLineitems = value;
                NotifyOfPropertyChange(() => OrderLineItems);
            }
        }

        private void LoadLineItems()
        {
            OrderLineItems = new ObservableCollection<OrderLineItemViewModel>();
            foreach (LineItem li in orderItem.LineItems)
            {
                OrderLineItemViewModel orderLineItemViewModel = new OrderLineItemViewModel(li);
                OrderLineItems.Add(orderLineItemViewModel);
            }
        }
    }
}
