﻿using Caliburn.Micro;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Controls.Validation;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter.DataModel;
using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Profile
{
    [Export(typeof(EditProfileViewModel))]
    public class EditProfileViewModel : ValidatingScreen<EditProfileViewModel>
    {
        private readonly IEventAggregator eventAggregator;
        private ProfileItem profile;

        [ImportingConstructor]
        public EditProfileViewModel(IEventAggregator evt, ProfileItem item)
        {
            eventAggregator = evt;
            profile = item;
            DisplayName = "Edit Profile";
        }

        //[RequiredEx(ErrorMessage = "Required")]
        public string CompanyName
        {
            get
            {
                return profile.CompanyName;
            }
            set
            {
                profile.CompanyName = value;
                NotifyOfPropertyChange(() => CompanyName);
            }
        }

        //[RequiredEx(ErrorMessage = "Required")]
        public string Email
        {
            get
            {
                return profile.Email;
            }
            set
            {
                profile.Email = value;
                NotifyOfPropertyChange(() => Email);
            }
        }

        //[RequiredEx(ErrorMessage = "Required")]
        public string FirstName
        {
            get
            {
                return profile.DefaultAddress.FirstName;
            }
            set
            {
                profile.DefaultAddress.FirstName = value;
                NotifyOfPropertyChange(() => FirstName);
            }
        }

        //[RequiredEx(ErrorMessage = "Required")]
        public string LastName
        {
            get
            {
                return profile.DefaultAddress.LastName;
            }
            set
            {
                profile.DefaultAddress.LastName = value;
                NotifyOfPropertyChange(() => LastName);
            }
        }

        //[RequiredEx(ErrorMessage = "Required")]
        public string PhoneNumber
        {
            get
            {
                return profile.DefaultAddress.PhoneNumber;
            }
            set
            {
                profile.DefaultAddress.PhoneNumber = value;
                NotifyOfPropertyChange(() => PhoneNumber);
            }
        }

        //[RequiredEx(ErrorMessage = "Required")]
        public string Address1
        {
            get
            {
                return profile.DefaultAddress.AddressLine1;
            }
            set
            {
                profile.DefaultAddress.AddressLine1 = value;
                NotifyOfPropertyChange(() => Address1);
            }
        }

        public string Address2
        {
            get
            {
                return profile.DefaultAddress.AddressLine2;
            }
            set
            {
                profile.DefaultAddress.AddressLine2 = value;
                NotifyOfPropertyChange(() => Address2);
            }
        }

        //[RequiredEx(ErrorMessage = "Required")]
        public string City
        {
            get
            {
                return profile.DefaultAddress.City;
            }
            set
            {
                profile.DefaultAddress.City = value;
                NotifyOfPropertyChange(() => City);
            }
        }

        //[RequiredEx(ErrorMessage = "Required")]
        public string State
        {
            get
            {
                return profile.DefaultAddress.Region;
            }
            set
            {
                profile.DefaultAddress.Region = value;
                NotifyOfPropertyChange(() => State);
            }
        }

        //[RequiredEx(ErrorMessage = "Required")]
        public string ZipCode
        {
            get
            {
                return profile.DefaultAddress.PostalCode;
            }
            set
            {
                profile.DefaultAddress.PostalCode = value;
                NotifyOfPropertyChange(() => ZipCode);
            }
        }

        public string Country
        {
            get
            {
                return profile.DefaultAddress.Country;
            }
        }

        public void Save()
        {
            TryClose(true);
        }

        public void Cancel()
        {
            TryClose(false);
        }
    }
}
