﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter.DataModel;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Profile
{
    [Export(typeof(ProfileListViewModel))]
    public class ProfileListViewModel : Screen
    {
        private IWindowManager windowManager;
        private IEventAggregator eventAggregator;
        private Profiles profiles = null;
        private ObservableCollection<ProfileViewModel> profileModels = null;
        private readonly BackgroundWorker worker = new BackgroundWorker();
        private Visibility showLoadIndicator = Visibility.Collapsed;
        private string customerId = null;
        private string tenantId = null;

        [ImportingConstructor]
        public ProfileListViewModel(IWindowManager winmgr, IEventAggregator evt, string customerTenantId, string customerCid)
        {
            windowManager = winmgr;
            eventAggregator = evt;
            customerId = customerCid;
            tenantId = customerTenantId;
            worker.DoWork += Worker_DoWork;
            worker.RunWorkerCompleted += Worker_RunWorkerCompleted;
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }

        public Visibility ShowLoadIndicator
        {
            get
            {
                return showLoadIndicator;
            }
            set
            {
                showLoadIndicator = value;
                NotifyOfPropertyChange(() => ShowLoadIndicator);
            }
        }

        private void Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ShowLoadIndicator = Visibility.Collapsed;
            LoadProfiles();
        }

        private void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            AuthTokenCacheInfo cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(App.CurrentAppContext.Settings.ResellerTenantId, App.CurrentAppContext.Settings.PartnerCenterAppId, App.CurrentAppContext.Settings.PartnerCenterAppSecret, Microsoft.Samples.CloudSolutionProviderManagement.Constants.AzureADGraphAPIUrl);
            PartnerCenterAuthToken authToken = new PartnerCenterAuthToken(cacheInfo.AccessToken, cacheInfo.ExpiresOn);

            Task<PartnerCenterAuthToken> caTokenTask = PartnerCenterManager.GetCustomerAgentToken(authToken, customerId, handler);
            caTokenTask.Wait();
            handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            Task<Profiles> profilesTask = PartnerCenterManager.GetProfiles(caTokenTask.Result, handler, customerId);
            profilesTask.Wait();
            profiles = profilesTask.Result;
        }

        public ObservableCollection<ProfileViewModel> ProfileModels
        {
            get
            {
                return profileModels;
            }
            set
            {
                profileModels = value;
                NotifyOfPropertyChange(() => ProfileModels);
            }
        }

        public void LoadProfiles()
        {
            ProfileModels = new ObservableCollection<ProfileViewModel>();

            if (profiles != null)
            {
                foreach (ProfileItem item in profiles.Items)
                {
                    ProfileModels.Add(new ProfileViewModel(windowManager, eventAggregator, item));
                }
            }
        }
    }
}
