﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.AzureActiveDirectory.Application;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.AzureActiveDirectory.Group;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.AzureActiveDirectory.ServicePrincipal;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.AzureActiveDirectory.Tenant;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.AzureActiveDirectory.User;
using Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Profile;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter.DataModel;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter
{
    [Export(typeof(ResellerViewModel))]
    public class ResellerViewModel : Conductor<object>
    {
        private readonly IWindowManager windowManager;
        private readonly IEventAggregator eventAggregator;
        private readonly BackgroundWorker worker = new BackgroundWorker();
        private Visibility showLoadIndicator = Visibility.Collapsed;
        private Reseller reseller;
        private string tenantId;
        private bool isTest;
        private LoginType authType;

        [ImportingConstructor]
        public ResellerViewModel(IWindowManager winMgr, IEventAggregator evt, LoginType loginType)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            authType = loginType;
            worker.DoWork += worker_DoWork;
            worker.RunWorkerCompleted += worker_RunWorkerCompleted;
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }

        void worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ShowLoadIndicator = Visibility.Collapsed;
            LoadResellerInfo();
        }

        void worker_DoWork(object sender, DoWorkEventArgs e)
        {
            var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            AuthTokenCacheInfo cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(App.CurrentAppContext.Settings.ResellerTenantId, App.CurrentAppContext.Settings.PartnerCenterAppId, App.CurrentAppContext.Settings.PartnerCenterAppSecret, Microsoft.Samples.CloudSolutionProviderManagement.Constants.AzureADGraphAPIUrl);
            PartnerCenterAuthToken authToken = new PartnerCenterAuthToken(cacheInfo.AccessToken, cacheInfo.ExpiresOn);

            Task<PartnerCenterAuthToken> saTokenTask = PartnerCenterManager.GetSalesAgentToken(authToken, handler);
            saTokenTask.Wait();
            handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            Task<Reseller> resellerTask = PartnerCenterManager.GetReseller(saTokenTask.Result, handler, App.CurrentAppContext.Settings.ResellerTenantId);
            resellerTask.Wait();
            reseller = resellerTask.Result;
        }
        

        public Visibility ShowLoadIndicator
        {
            get
            {
                return showLoadIndicator;
            }
            set
            {
                showLoadIndicator = value;
                NotifyOfPropertyChange(() => ShowLoadIndicator);
            }
        }

        public string TenantId
        {
            get
            {
                return tenantId;
            }
            set
            {
                tenantId = value;
                NotifyOfPropertyChange(() => TenantId);
            }
        }

        public bool IsTest
        {
            get
            {
                return isTest;
            }
            set
            {
                isTest = value;
                NotifyOfPropertyChange(() => IsTest);
            }
        }

        public void LoadResellerInfo()
        {
            if (reseller != null)
            {
                TenantId = reseller.Id;
                IsTest = reseller.IsTest;
            }
        }

        public void Profile()
        {
            ActivateItem(new ProfileListViewModel(windowManager, eventAggregator, TenantId, TenantId));
        }

        public void Users()
        {
            ActivateItem(new UserListViewModel(windowManager, eventAggregator, TenantId, authType));
        }

        public void Groups()
        {
            ActivateItem(new GroupListViewModel(windowManager, eventAggregator, TenantId, authType));
        }

        public void Applications()
        {
            ActivateItem(new ApplicationListViewModel(windowManager, eventAggregator, TenantId, authType));
        }

        public void ServicePrincipals()
        {
            ActivateItem(new ServicePrincipalListViewModel(windowManager, eventAggregator, TenantId, authType));
        }

        public void TenantDetails()
        {
            ActivateItem(new TenantViewModel(windowManager, eventAggregator, TenantId, authType));
        }
    }
}
