﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter.DataModel;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Subscription
{
    [Export(typeof(SubscriptionListViewModel))]
    public class SubscriptionListViewModel : Screen
    {
        private IWindowManager windowManager;
        private IEventAggregator eventAggregator;
        private Subscriptions subscriptions = null;
        private ObservableCollection<SubscriptionViewModel> subscriptionModels = null;
        private readonly BackgroundWorker worker = new BackgroundWorker();
        private Visibility showLoadIndicator = Visibility.Collapsed;
        private string customerId = null;
        private string rcid = null;
        private string customerTenantId = null;
        private LoginType authType;

        [ImportingConstructor]
        public SubscriptionListViewModel(IWindowManager winMgr, IEventAggregator evt, string tenantId, string resellerCid, string customerCid, LoginType loginType)
        {
            windowManager = winMgr;
            eventAggregator = evt;
            customerId = customerCid;
            rcid = resellerCid;
            customerTenantId = tenantId;
            authType = loginType;
            worker.DoWork += Worker_DoWork;
            worker.RunWorkerCompleted += Worker_RunWorkerCompleted;
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }

        public Visibility ShowLoadIndicator
        {
            get
            {
                return showLoadIndicator;
            }
            set
            {
                showLoadIndicator = value;
                NotifyOfPropertyChange(() => ShowLoadIndicator);
            }
        }

        private void Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ShowLoadIndicator = Visibility.Collapsed;
            LoadSubscriptions();
        }

        private void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            AuthTokenCacheInfo cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(App.CurrentAppContext.Settings.ResellerTenantId, App.CurrentAppContext.Settings.PartnerCenterAppId, App.CurrentAppContext.Settings.PartnerCenterAppSecret, Microsoft.Samples.CloudSolutionProviderManagement.Constants.AzureADGraphAPIUrl);
            PartnerCenterAuthToken authToken = new PartnerCenterAuthToken(cacheInfo.AccessToken, cacheInfo.ExpiresOn);

            Task<PartnerCenterAuthToken> saTokenTask = PartnerCenterManager.GetSalesAgentToken(authToken, handler);
            saTokenTask.Wait();
            handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
            Task<Subscriptions> subsTask = PartnerCenterManager.GetSubscriptions(saTokenTask.Result, handler, rcid, customerId);
            subsTask.Wait();
            subscriptions = subsTask.Result;
        }

        public ObservableCollection<SubscriptionViewModel> SubscriptionModels
        {
            get
            {
                return subscriptionModels;
            }
            set
            {
                subscriptionModels = value;
                NotifyOfPropertyChange(() => SubscriptionModels);
            }
        }

        public void LoadSubscriptions()
        {
            SubscriptionModels = new ObservableCollection<SubscriptionViewModel>();

            if (subscriptions != null)
            {
                foreach (SubscriptionItem item in subscriptions.Items)
                {
                    SubscriptionModels.Add(new SubscriptionViewModel(windowManager, eventAggregator, customerTenantId, item, authType));
                }
            }
        }

        public void Reload()
        {
            worker.RunWorkerAsync();
            ShowLoadIndicator = Visibility.Visible;
        }
    }
}
