﻿using Caliburn.Micro;
using Microsoft.Samples.AuthenticationManagement;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Utility;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter;
using Microsoft.Samples.CloudSolutionProviderManagement.PartnerCenter.DataModel;
using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels.PartnerCenter.Subscription
{
    [Export(typeof(UpdateSubscriptionFriendlyNameViewModel))]
    public class UpdateSubscriptionFriendlyNameViewModel : Screen
    {
        private readonly IEventAggregator eventAggregator;
        private string resellerCid;
        private string subscriptionId;
        private string friendlyName;

        [ImportingConstructor]
        public UpdateSubscriptionFriendlyNameViewModel(IEventAggregator evt, string resellerId, string subId)
        {
            eventAggregator = evt;
            resellerCid = resellerId;
            subscriptionId = subId;
            DisplayName = "Update Subscription Friendly Name";
        }

        public string FriendlyName
        {
            get
            {
                return friendlyName;
            }
            set
            {
                friendlyName = value;
                NotifyOfPropertyChange(() => FriendlyName);
            }
        }

        public async void Save()
        {
            UpdateSubscriptionFriendlyName updateName = new UpdateSubscriptionFriendlyName();
            updateName.FriendlyName = FriendlyName;

            try
            {
                var handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
                AuthTokenCacheInfo cacheInfo = App.CurrentAppContext.AuthManager.GetTokenBySpn(App.CurrentAppContext.Settings.ResellerTenantId, App.CurrentAppContext.Settings.PartnerCenterAppId, App.CurrentAppContext.Settings.PartnerCenterAppSecret, Microsoft.Samples.CloudSolutionProviderManagement.Constants.AzureADGraphAPIUrl);
                PartnerCenterAuthToken authToken = new PartnerCenterAuthToken(cacheInfo.AccessToken, cacheInfo.ExpiresOn);

                PartnerCenterAuthToken saToken = await PartnerCenterManager.GetSalesAgentToken(authToken, handler);

                handler = new HttpLoggingHandler(eventAggregator, new HttpClientHandler(), true);
                CloudSolutionProviderManagement.PartnerCenter.DataModel.SubscriptionItem item = await PartnerCenterManager.UpdateSubscriptionFriendlyName(saToken, handler, App.CurrentAppContext.Settings.ResellerTenantId, subscriptionId, updateName);
                TryClose(true);
            }
            catch (Exception ex)
            {
                Logger.ErrorLine(eventAggregator, "{0} {1}", ex.Message, ex.StackTrace);
                MessageBox.Show(ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        public void Cancel()
        {
            TryClose(false);
        }
    }
}
