﻿using Caliburn.Micro;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Models;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Models.Events;
using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using System.Xml.Serialization;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels
{
    [Export(typeof(SampleConfigurationViewModel))]
    public class SampleConfigurationViewModel : Screen
    {
        private string tenantDomainName;
        private string tenantId;
        private string partnercenterAppId;
        private string partnercenterAppSecret;
        private string clientId;
        private string managementAppId;
        private string managementAppSecret;
        private IEventAggregator _events;

        ConfigurationSettings config;
        XmlSerializer configSerializer = new XmlSerializer(typeof(ConfigurationSettings));

        [ImportingConstructor]
        public SampleConfigurationViewModel(IEventAggregator events)
        {
            _events = events;
            if (File.Exists(Path.Combine(Path.GetDirectoryName(Process.GetCurrentProcess().MainModule.FileName), "CSPSamples.Config")))
            {
                using (FileStream fs = new FileStream(Path.Combine(Path.GetDirectoryName(Process.GetCurrentProcess().MainModule.FileName), "CSPSamples.Config"), FileMode.Open, FileAccess.Read))
                {
                    XmlReader reader = XmlReader.Create(fs);
                    config = (ConfigurationSettings)configSerializer.Deserialize(reader);
                }
                App.CurrentAppContext.Settings = config;
                TenantDomainName = config.ResellerTenantDomainName;
                TenantId = config.ResellerTenantId;
                PartnerCenterAppId = config.PartnerCenterAppId;
                PartnerCenterAppSecret = config.PartnerCenterAppSecret;
                ManagementAppId = config.ManagementAppId;
                ManagementAppSecret = config.ManagementAppSecret;
                ClientId = config.ClientId;
            }
            this.DisplayName = "Cloud Solution Provider Samples Configuration";
        }

        public string Description
        {
            get
            {
                return "Please specify the following configuration settings:";
            }
        }
        public string TenantDomainName
        {
            get
            {
                return tenantDomainName;
            }
            set
            {
                tenantDomainName = value;
                NotifyOfPropertyChange(() => TenantDomainName);
            }
        }

        public string TenantId
        {
            get
            {
                return tenantId;
            }
            set
            {
                tenantId = value;
                NotifyOfPropertyChange(() => TenantId);
            }
        }

        public string PartnerCenterAppId
        {
            get
            {
                return partnercenterAppId;
            }
            set
            {
                partnercenterAppId = value;
                NotifyOfPropertyChange(() => PartnerCenterAppId);
            }
        }

        public string PartnerCenterAppSecret
        {
            get
            {
                return partnercenterAppSecret;
            }
            set
            {
                partnercenterAppSecret = value;
                NotifyOfPropertyChange(() => PartnerCenterAppSecret);
            }
        }

        public string ClientId
        {
            get
            {
                return clientId;
            }
            set
            {
                clientId = value;
                NotifyOfPropertyChange(() => ClientId);
            }
        }

        public string ManagementAppId
        {
            get
            {
                return managementAppId;
            }
            set
            {
                managementAppId = value;
                NotifyOfPropertyChange(() => ManagementAppId);
            }
        }

        public string ManagementAppSecret
        {
            get
            {
                return managementAppSecret;
            }
            set
            {
                managementAppSecret = value;
                NotifyOfPropertyChange(() => ManagementAppSecret);
            }
        }

        public void Save()
        {
            if (config == null)
                config = new ConfigurationSettings();
            config.ResellerTenantId = TenantId;
            config.ResellerTenantDomainName = TenantDomainName;
            config.PartnerCenterAppId = PartnerCenterAppId;
            config.PartnerCenterAppSecret = PartnerCenterAppSecret;
            config.ClientId = ClientId;
            config.ManagementAppId = ManagementAppId;
            config.ManagementAppSecret = ManagementAppSecret;
            SaveConfiguration(configSerializer, config);

            _events.PublishOnCurrentThread(new SavedConfigurationSettingEvent());
            TryClose();
        }

        private void SaveConfiguration(XmlSerializer configSerializer, ConfigurationSettings config)
        {
            XmlWriterSettings settings = new XmlWriterSettings();
            settings.Indent = true;
            XmlWriter writer = XmlWriter.Create(Path.Combine(Path.GetDirectoryName(Process.GetCurrentProcess().MainModule.FileName), "CSPSamples.Config"), settings);
            writer.WriteComment("File generated by CSPSamples " + System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString() + " on " + DateTime.Now.ToShortDateString());
            configSerializer.Serialize(writer, config);
            writer.Close();
            App.CurrentAppContext.Settings = config;
        }
    }
}
