﻿using Caliburn.Micro;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Models;
using Microsoft.Samples.CloudSolutionProviderControlPanel.Models.Events;
using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Xml;
using System.Xml.Serialization;

namespace Microsoft.Samples.CloudSolutionProviderControlPanel.ViewModels
{
    [Export(typeof(IShell))]
    public class ShellViewModel : Conductor<object>, IShell, IHandle<object>
    {
        private readonly IWindowManager windowManager;
        private readonly IEventAggregator evtAggregator;
        private bool isConfigured = false;
        ConfigurationSettings config;
        XmlSerializer configSerializer = new XmlSerializer(typeof(ConfigurationSettings));

        [ImportingConstructor]
        public ShellViewModel(IWindowManager winManager, IEventAggregator evt)
        {
            this.windowManager = winManager;
            this.evtAggregator = evt;
            this.evtAggregator.Subscribe(this);
            this.DisplayName = "Cloud Solution Provider Samples";
        }

        protected override void OnViewLoaded(object view)
        {
            base.OnViewLoaded(view);
            LoadConfigurationSettings();
        }

        public string SampleDescription
        {
            get
            {
                return "This sample application covers the key use cases for demonstrating how to integrate with various Microsoft APIs.";
            }
        }

        public string CSPAdminSampleDescription
        {
            get
            {
                return "This sample demonstrates Cloud Solution Provider administrative capabilities, such as managing customers, orders, subscriptions via Partner Center APIs, and various Microsoft services via Azure Reosurce Management APIs, etc. The sample impersonalizes under user security context using User Authorization Code option.";
            }
        }

        public string CSPCustomerSampleDescription
        {
            get
            {
                return "This sample demonstrates customer manages their services provisioned under Cloud Solution Provider. The sample impersonalizes under the logged in customer user account.";
            }
        }

        public string CSPServerSampleDescription
        {
            get
            {
                return "This sample demonstrates how to perform various administrative operations in an automated fashion without any user interaction. The sample uses application credential to access various Microsoft APIs.";
            }
        }

        public string ConfigureSampleDescription
        {
            get
            {
                return "Configure Sample Configurations by providing Cloud Sloution Provider tenant information and application information.";
            }
        }

        public void LaunchCSPAdminSample()
        {
            windowManager.ShowWindow(new CSPAdminViewModel(windowManager, evtAggregator), null, null);
        }

        public void LaunchCSPCustomerSample()
        {
            windowManager.ShowWindow(new CSPCustomerViewModel(windowManager, evtAggregator), null, null);
        }

        public void LaunchCSPServerSample()
        {
            windowManager.ShowWindow(new CSPServerViewModel(windowManager, evtAggregator), null, null);
        }


        public void ConfigureSample()
        {
            windowManager.ShowWindow(new SampleConfigurationViewModel(evtAggregator), null, null);
        }

        public void ShowLog()
        {
            windowManager.ShowWindow(new LogViewModel(evtAggregator), null, null);
        }

        public void LoadConfigurationSettings()
        {
            if (File.Exists(Path.Combine(Path.GetDirectoryName(Process.GetCurrentProcess().MainModule.FileName), "CSPSamples.Config")))
            {
                using (FileStream fs = new FileStream(Path.Combine(Path.GetDirectoryName(Process.GetCurrentProcess().MainModule.FileName), "CSPSamples.Config"), FileMode.Open, FileAccess.Read))
                {
                    XmlReader reader = XmlReader.Create(fs);
                    config = (ConfigurationSettings)configSerializer.Deserialize(reader);
                }
                App.CurrentAppContext.Settings = config;
                IsConfigured = true;
            }
            else
            {
                IsConfigured = false;

                MessageBox.Show("Please specify your configuration settings to use this sample!", "Attention", MessageBoxButton.OK, MessageBoxImage.Information);
            }
        }

        public override void CanClose(Action<bool> callback)
        {
            base.CanClose(callback);
            App.Current.Shutdown();
        }

        public bool IsConfigured
        {
            get
            {
                return isConfigured;
            }
            set
            {
                isConfigured = value;
                NotifyOfPropertyChange(() => IsConfigured);
            }
        }

        public void Handle(object message)
        {
            if (message is SavedConfigurationSettingEvent)
            {
                IsConfigured = true;
            }
        }
    }
}
