﻿#Switch Azure to ARM mode
Switch-AzureMode "AzureResourceManager"
#Adding Azure Account
Add-AzureAccount -Tenant "" #Inse3rt TenantID
Get-azuresubscription | fl
Select-AzureSubscription -SubscriptionId "" #Insert Subscripion ID
#Register ASR provider
Register-AzureProvider -ProviderNamespace Microsoft.SiteRecovery
#Confirm Azure Provider is registered
Get-AzureProvider -ProviderNamespace Microsoft.SiteRecovery

#Initialize variables - Modify this with your parameters
$VaultName = "CSPgosemrASRVault"
$OutputPathForSettingsFile= "C:\ASR\New"
$ResourceGroupName= "ASRtest"
$Location= "West Europe"


#Create resource group
New-AzureResourceGroup -Name $ResourceGroupName -Location $Location

#Create ASR Vault
New-AzureSiteRecoveryVault -Name CSPgosemrASRVault -ResouceGroupName $ResourceGroupName -Location $Location 

#To access the Azure Site Recovery vault  in your Azure subscription use the Get-AzureSiteRecoveryVault cmdlet
$Vault = Get-AzureSiteRecoveryVault -ResouceGroupName $ResourceGroupName | where { $_.Name -eq $VaultName}

#You can download the vault settings file to a local machinestorage using Get-AzureSiteRecoveryVaultSettingsFile cmdlet
$VaultSetingsFile = Get-AzureSiteRecoveryVaultSettingsFile -Vault $Vault -Path $OutputPathForSettingsFile

#Setup the context for the Azure Site Recovery vault using Import-AzureSiteRecoveryVaultSettingsFile cmdlet. Post this all cmdlets will execute under given vault context. 
Import-AzureSiteRecoveryVaultSettingsFile -Path $VaultSetingsFile.FilePath
#Temp
Import-AzureSiteRecoveryVaultSettingsFile -Path "C:\ASR\New\CSPgosemrASRVault_2015-08-29T13-11-19.VaultCredentials"


#Register VMM server to subscription through PS (Optional)
$BinPath = $env:SystemDrive+"\Program Files\Microsoft System Center 2012 R2\Virtual Machine Manager\bin"
pushd $BinPath
$encryptionFilePath = "C:\temp\"
.\DRConfigurator.exe /r /Credentials $VaultSettingFilePath /vmmfriendlyname $env:COMPUTERNAME /dataencryptionenabled $encryptionFilePath /startvmmservice

################### Configuring ASR Settings ############################

#List protection containers (Clouds)
Get-AzureSiteRecoveryProtectionContainer

#Initialize variables from the output.
$PrimaryCloudName= "DemoCloud"
$RecoveryCloudName= "DRCloud"
$ProtectionProfileName= "ASRtestProfileSlow" #Name of the Protection Profile that you will create in next step

#To access the primary and secondary protection container execute Get-AzureSiteRecoveryProtectionContainer cmdlet.
$PrimaryContainer = Get-AzureSiteRecoveryProtectionContainer -FriendlyName  $PrimaryCloudName 
$RecoveryContainer = Get-AzureSiteRecoveryProtectionContainer -FriendlyName  $RecoveryCloudName 

#Create a protection profile for protection by running the New-AzureSiteRecoveryProtectionProfile cmdlet
New-AzureSiteRecoveryProtectionProfile -Name $ProtectionProfileName -ReplicationProvider HyperVReplica -ReplicationMethod Online -ReplicationFrequencyInSeconds 300 -RecoveryPoints 1 -ApplicationConsistentSnapshotFrequencyInHours 0 -ReplicationPort 8084 -Authentication Kerberos -AllowReplicaDeletion 
$ProtectionProfile = Get-AzureSiteRecoveryProtectionProfile -Name $ProtectionProfileName
#To complete the configuration of the protection container start the association of the protection container and profile using Start-AzureSiteRecoveryProtectionProfileAssociationJob cmdlet (note the JOB Name) 
Start-AzureSiteRecoveryProtectionProfileAssociationJob -ProtectionProfile $ProtectionProfile -PrimaryProtectionContainer $PrimaryContainer -RecoveryProtectionContainer $RecoveryContainer -Verbose
Start-AzureSiteRecoveryProtectionProfileDissociationJob -ProtectionProfile $ProtectionProfile -PrimaryProtectionContainer $PrimaryContainer -RecoveryProtectionContainer $RecoveryContainer


#Check completion of job (insert your job Name from previous step)
$Job=Get-AzureSiteRecoveryJob -Name 4115f7a9-cc30-4f75-813a-1e6e9692f7d6 #replace with your job ID
$Job.State | fl

############ Confugure Protection ##################

#Intialize variables for Protected Entities
$VMName=  "DC01"

#Select the virtual Machine on which protection need to be enabled using Get-AzureSiteRecoveryProtectionEntity cmdlet
$VM = Get-AzureSiteRecoveryProtectionEntity -ProtectionContainer $PrimaryContainer -FriendlyName $VMName 
#Enable the protection on the virtual machine using the Set-AzureSiteRecoveryProtectionEntity cmdlet
Set-AzureSiteRecoveryProtectionEntity -ProtectionEntity $VM -Protection Enable
$JobProtection=Get-AzureSiteRecoveryJob -Name 46998979-ce20-429c-a3a2-e400b5dd414a #replace with your job ID
$JobProtection.Errors | fl

$JobProtection=Get-AzureSiteRecoveryJob -Name 46998979-ce20-429c-a3a2-e400b5dd414a

PS C:\Users\gosemr\OneDrive - Microsoft\Readiness and IP\CSP> $JobProtection.Errors. | fl
                $JobProtection.Errors[0].ServiceErrorDetails
                $JobProtection.Errors[0].ProviderErrorDetails

############Test Failover###############

#Select the virtual Machine on which protection need to be enabled using Get-AzureSiteRecoveryProtectionEntity cmdlet
$VM = Get-AzureSiteRecoveryProtectionEntity -ProtectionContainer $PrimaryContainer -FriendlyName  $VMName 
#To start the test failover of the VM use Start-AzureSiteRecoveryTestFailoverJob cmdlet 
$ currentJob = Start-AzureSiteRecoveryTestFailoverJob -ProtectionEntity $VM -Direction PrimaryToRecovery
#Once the VM functionality is been validated you can be complete the failover using below cmdlet 
Resume-AzureSiteRecoveryJob -Id $currentJob.Name
